#include <iostream>
#include <iomanip>
#include <string>

#include "Y86_H.h"

// Names of Register
const char *Regs[8]=
  { "eax", "ecx", "edx", "ebx", "esp", "ebp", "esi", "edi" };

unsigned char program[]=
  "\x29\xF6"          // 00000000 sub esi,esi
  "\x29\xC0"          // 00000002 sub eax,eax
  "\x29\xDB"          // 00000004 sub ebx,ebx
  "\x8B\x56\x17"      // 00000006 mov edx,[esi+0x17]
  "\x01\xD0"          // 00000009 add eax,edx
  "\x01\xC3"          // 0000000B add ebx,eax
  "\x89\xC1"          // 0000000D mov ecx,eax
  "\x8B\x56\x1B"      // 0000000F mov edx,[esi+0x1b]
  "\x29\xD1"          // 00000012 sub ecx,edx
  "\x75\xF0"          // 00000014 jnz 0x6
  "\xF4"              // 00000016 hlt
  "\x01\x00\x00\x00"  // 00000017 dd 1
  "\x0a\x00\x00\x00"; // 0000001B dd 10


void cpu_Y86_H::init()
{
  IP_=0;

  for(unsigned i=0; i<100; i++)
    MEM_[i]=0;

  for(unsigned j=0; j<8; j++)
    R_[j]=0;
  
  ZF_=false;

  for(unsigned i=0; i<sizeof(program); i++)
    MEM_[i]=program[i];
}

void cpu_Y86_H::show() 
{
  std::cout << "IP=" << std::hex << IP_ << std::endl;

  for(unsigned i=0; i<8; i++)
    std::cout << Regs[i] << "=" << std::hex << std::setw(9) << R_[i] << " ";

  std::cout << "ZF=" << ZF_ << std::endl;

  std::cout << "MEM:";

  for(unsigned i=0; i<32; i++)
  {
    std::cout << std::hex << std::setw(3) << (unsigned)MEM_[i];
    if((i&0xf)==0xf) 
      std::cout << std::endl << "     ";
  }
  std::cout << std::endl;
}

void cpu_Y86_H::step() 
{
  unsigned ea;
  unsigned I0=MEM_[IP_];
  unsigned I1=MEM_[IP_+1];
  unsigned I2=MEM_[IP_+2];
  IP_+=1;
  
  switch(I0) 
  {
    case MRmov:
      if(mod(I1)==1) // move from memory to register
      {
				IP_+=2;
				std::cout << "M->Rmov " << Regs[RS(I1)] << ", [ESI+0x" << EXTEND8(I2) << "]" << std::endl;
				ea=R_[6]+EXTEND8(I2);
				R_[RS(I1)]=MEM_[ea]|(MEM_[ea+1]>>8)|(MEM_[ea+2]>>16)|(MEM_[ea+3]>>24);
      }
      else
				std::cout << "Mode not under current" << std::endl;
      break;
    case RMmov:
      if(mod(I1)==1) // move from register to memory
      {
				IP_+=2;
				ea=R_[6]+EXTEND8(I2);
				std::cout << "R->Mmov [ESI+0x" << EXTEND8(I2) << "], " << Regs[RS(I1)] << std::endl;
				MEM_[ea+0]=(R_[RS(I1)]&0xff);
				MEM_[ea+1]=(R_[RS(I1)]&0xff00)>>8;
				MEM_[ea+2]=(R_[RS(I1)]&0xff0000)>>16;
				MEM_[ea+3]=(R_[RS(I1)]&0xff000000)>>24;
      }
      else if(mod(I1)==3) // move from register to register
      {
				IP_+=1;
				std::cout << "R->Rmov " << Regs[RD(I1)] << ", " << Regs[RS(I1)] << std::endl;
				R_[RD(I1)]=R_[RS(I1)];
      }
      else
				std::cout << "Mode not under current" << std::endl;
      break;
    case add:
      IP_+=1;
      std::cout << "add " << Regs[RD(I1)] << ", " << Regs[RS(I1)] << std::endl;
      R_[RD(I1)]+=R_[RS(I1)];
      ZF_=(R_[RD(I1)]==0); 
      break;
    case sub:
      IP_+=1;
      std::cout << "sub " << Regs[RD(I1)] << ", " << Regs[RS(I1)] << std::endl;
      R_[RD(I1)]-=R_[RS(I1)];
      ZF_=(R_[RD(I1)]==0); 
      break;
    case jnz: // jump if not zero
      IP_+=1;
      std::cout << "jnz 0x" << IP_+EXTEND8(I1) << std::endl;
      if(!ZF_) IP_+=EXTEND8(I1);
      break;     
    case hlt: // halt
      std::cout << "hlt" << std::endl;
      exit(0);
    default:
      std::cout << "Unknown Instruction" << std::endl;
  }
}

void cpu_Y86_H::run() {
  init();

  while(true) 
  {
    show();

    std::string s;
    if(!std::getline(std::cin, s)) break;
    if(s=="q") break;
    
    step();
  }
}

