#include <iostream>
#include "Y86_L.h"

void cpu_Y86_L::stage_0_pre_assign()
{
  sc_uint<5> ue_stage_0;
  while(true)
  {
    ue_stage_0.range(4, 1)=full.range(4, 1);
    ue_stage_0[0]=full[5];

    ue.write(ue_stage_0);
  }
}

  // stage 0
void cpu_Y86_L::stage_0()
{
  if(rst_.read())
  {
    for(unsigned i=1; i<=5; i++)
      full[i]=false;
  }
  else
  {
    for(unsigned i=5; i>0; i--)
      full[i]=ue.read()[i-1];
  }
}

  // stage 1 IF method
void cpu_Y86_L::process_IF()
{
  if(ue.read()[0])
    IR=bus_in_.read();
}

  // stage 2 ID method
void cpu_Y86_L::process_ID_pre_assign()
{
  sc_uint<32> v_stage_2;

  while(true)
  {
    opcode.write(IR.range(7,0));
    mod.write(IR.range(15,14));

    load.write((opcode.read()==0x8b) && (mod.read()==1));
    move.write((opcode.read()==0x89) && (mod.read()==3));
    store.write((opcode.read()==0x89) && (mod.read()==1));
    memory.write(load.read()||store.read());
    add.write(opcode.read()==0x01);
    sub.write(opcode.read()==0x29);
    halt.write(opcode.read()==0xF4);
    aluop.write((add.read())||(sub.read()));
    jnez.write(opcode.read()==0x75);

    RD.write(IR.range(10,8));
    RS.write(IR.range(13,11));

    if(memory.read()==true)
      Aad.write(6);
    else
      Aad.write(RD.read());

    Bad.write(RS.read());
    
    for(unsigned i=31; i>=8; i--)
      v_stage_2[i]=IR[15];
    for(unsigned i=7; i>=0; i--)
      v_stage_2[i]=IR[i+8];
    distance.write(v_stage_2);

    for(unsigned i=31; i>=8; i--)
      v_stage_2[i]=IR[23];
    for(unsigned i=7; i>=0; i--)
      v_stage_2[i]=IR[i+16];
    displacement.write(v_stage_2);

    btaken.write(jnez.read()&&(ZF==false));

    if(memory.read())
      length.write(3);
    else 
    {
      if(aluop.read()||move.read()||jnez.read())
	length.write(2);
      else
	length.write(1);
    }
  }
}

void cpu_Y86_L::process_ID()
{
  if(rst_.read())
    IP=0x0000;
  else if(ue.read()[1])
  {
    A=Aop.read();
    B=Bop.read();

    if(halt.read()==false)
      IP=IP+length.read();

    if(btaken.read()==true)
      IP=IP+distance.read();
  }
}

  // stage 3 EX method
void cpu_Y86_L::process_EX_pre_assign()
{
  while(true)
  {
    if(memory.read())
      ALU_op2.write(displacement.read());
    else
      if(sub.read())
	ALU_op2.write(B^0xFFFF);
      else
	ALU_op2.write(B);

    ALUout.write(A+ALU_op2.read()+sub.read());  
  }
}

void cpu_Y86_L::process_EX()
{  
  if(rst_.read())
    ZF=false;
  else if(ue.read()[2])
  {
    MAR=ALUout.read();
    C=(move.read())?B:ALUout.read();
    MDRw=B;
    if(aluop.read())
      ZF=(ALUout.read()==0x0000);
  }
}

 // stage 4 MEM method
void cpu_Y86_L::process_MEM()
{
  if(ue.read()[3]&&load.read())
    MDRr=bus_in_.read();

  if(ue.read()[3])
    bus_A_.write(MAR);
  else
  {
    if(ue.read()[0])
      bus_A_.write(IP);
    else
      bus_A_.write(0x0000);
  }

  bus_RE_.write(ue.read()[0]||(ue.read()[3]&&load.read()));
}

 // stage 4 MEM method
void cpu_Y86_L::process_WB_pre_assign()
{
  while(true)
  {
    Aop.write(*R[Aad.read()]);
    Bop.write(*R[Bad.read()]);

    bus_WE_.write(ue.read()[3] && store.read());
    bus_out_.write(MDRw);
  }
}

void cpu_Y86_L::process_WB()
{
  if(rst_.read()==true)
  {
    *R[0x00]=0x00;
    *R[0x01]=0x00;
    *R[0x02]=0x00;
    *R[0x03]=0x00;
    *R[0x04]=0x00;
    *R[0x05]=0x00;
    *R[0x06]=0x00;
    *R[0x07]=0x00;
  }
  else
  {
    if(ue.read()[4]==true) {
      if(aluop.read()||move.read()||load.read())
      {
	if(load.read()==true)
	  *R[RS.read()]=MDRr;
	else
	  *R[RD.read()]=C;
      }
    }
  }
}

void cpu_Y86_L::process_WB_aft_assign()
{
  while(true)
  {
    current_opcode_.write(opcode.read());
    eax.write(*R[0]);
    ecx.write(*R[1]);
    edx.write(*R[2]);
    ebx.write(*R[3]);
    esp.write(*R[4]);
    ebp.write(*R[5]);
    esi.write(*R[6]);
    edi.write(*R[7]);
    regs.write(IR.range(15, 8));
  }
}
