`timescale 1ns / 1ps

module memory(input clk, input rst,
              input [31:0] data_in, input [31:0] addr,
				  input RE, input WE, 
				  output [31:0] data_out,
				  output [31:0] magic_out);
				  
  localparam [9:0] magic_location = 'hFFFFFFF0;
  reg [31:0] magic_buffer;
  
  assign magic_out = magic_buffer;
  
  reg [7:0] bufr[3:0];
  reg [7:0] bufw[3:0];
  
  reg [2:0] inx;
  wire [9:0] b_addr;
  
  wire [7:0] mem_in;
  wire [7:0] mem_out;
  
  reg r_b, w_b;
  
  always @(posedge clk) begin
    if(rst) begin
	   inx=0; r_b=0; w_b=0;
    end 
	 else if(r_b) begin
		bufr[inx-1]=mem_out;
		inx=inx+1;
		if(inx==5) begin r_b=0; inx=0; end
	 end 
	 else if(w_b) begin
		inx=inx+1;
		if(inx==4) begin w_b=0; inx=0; end
    end 
	 else if(RE) begin
	   inx=1;
		r_b=1;
	 end
	 else if(WE) begin
	   inx=0;
		bufw[3]=data_in[31:24];
		bufw[2]=data_in[23:16];
		bufw[1]=data_in[15:8];
		bufw[0]=data_in[7:0];
		if(b_addr==magic_location) magic_buffer = data_in;
		w_b=1;
	 end
  end
  
  assign mem_in = bufw[inx];
  assign b_addr = addr+inx;
  
  onchip_memory mem(clk, mem_in, b_addr, RE || WE || r_b || w_b, WE || w_b, mem_out);
  
  assign data_out = { bufr[3], bufr[2], bufr[1], bufr[0] };

endmodule
