//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//-- This confidential and proprietary software may be used only as authorized 
//-- by a licensing agreement from ESD University of Verona - EDALab s.r.l.                      
//--                                                                            
//--                                                                           
//-- Title:             Multimedia System Design Developement
//-- Project:           V-CLIP 1.0
//--    
//-- 
//-- File:              adpcm_UT.h
//-- 
//-- Mod History: $Log: $
//                                                                            
//----------------------------------------------------------------------------
//-- RCS Version:  $Id: $                                           
//----------------------------------------------------------------------------

#include <systemc.h>

// DATA TYPE DEFINITIONS
typedef struct
{
	short len;
	short sample;
	char  result;
} DATA_TYPE_adpcm;

struct packet_adpcm {
	short* sbuf;
	int len;
};

// Added for the simulation of abstracted ADPCM module.
struct iostruct {
	sc_uint<16> len;
	sc_int<16> indata;
	char outdata;
	bool outdata_enable;
	bool indata_enable;
	bool len_enable;
};

typedef unsigned int ADDRESS_TYPE;

// CONSTANTS
#define ADPCM_ADDR   5
#define NSAMPLES   200
#define CLKTIME      1
#define TIMETYPE SC_NS

SC_MODULE(adpcm_UT) 
{
	
	// Local variables in the original SystemC RTL design.
	// Cannot be declared and initialized as class variables,
	// so we move them inside the original method.
	
	int flag_return;
	
	iostruct ioDataStruct;
	
	typedef enum 
	{
		RESET, PRE1, PRE2, STEP01, STEP0, 
		STEP1, STEP2, STEP3, STEP4, STEP5, STEP6
	}  state_adpcm;
	
	state_adpcm state;
	state_adpcm next_state;
	
	char c;
	char outputbuffer;
	
	sc_int<16> val;
	sc_uint<4> sign;
	sc_uint<4> delta;
	sc_uint<18> step;
	sc_uint<18> vpdiff;
	sc_uint<16> len_value;
	int valpred;
	int diff;
	int index;
	bool bufferstep;
	
	void adpcm_coder();
	void reset();
	void run();
	
	SC_CTOR(adpcm_UT)
	{
		SC_THREAD(run);
	}
};

// FUNCTIONALITY SIDE:
//****************
void adpcm_UT::adpcm_coder()
{
	
	int indexTable[16] = {
		-1, -1, -1, -1, 2, 4, 6, 8,
		-1, -1, -1, -1, 2, 4, 6, 8,
	};
	
	int stepsizeTable[89] = {
		7, 8, 9, 10, 11, 12, 13, 14, 16, 17,
		19, 21, 23, 25, 28, 31, 34, 37, 41, 45,
		50, 55, 60, 66, 73, 80, 88, 97, 107, 118,
		130, 143, 157, 173, 190, 209, 230, 253, 279, 307,
		337, 371, 408, 449, 494, 544, 598, 658, 724, 796,
		876, 963, 1060, 1166, 1282, 1411, 1552, 1707, 1878, 2066,
		2272, 2499, 2749, 3024, 3327, 3660, 4026, 4428, 4871, 5358,
		5894, 6484, 7132, 7845, 8630, 9493, 10442, 11487, 12635, 13899,
		15289, 16818, 18500, 20350, 22385, 24623, 27086, 29794, 32767
	};
	
	c = 0;		
	outputbuffer = 0;
	val = 0;	
	sign = 0;	
	delta = 0;	
	step = 0;	
	vpdiff = 0;	
	len_value = 0; 
	valpred = 0;
	diff = 0;	
	index = 0;	
	bufferstep = 0;
	
	ioDataStruct.outdata_enable = false;
	len_value = ioDataStruct.len;
	
	step = stepsizeTable[index];
	bufferstep = 1;
	
	val = ioDataStruct.indata;       // read new indata value
	diff = val - valpred;
	sign = (diff < 0) ? 8 : 0;
	if( sign ) 
		diff = (-diff);
	
	delta = 0;
	vpdiff = (step >> 3);
	if(diff >= (signed)step)
	{
		delta = 4;
		diff -= step;
		vpdiff += step;
	}
	
	step >>= 1;			
	if(diff >= (signed)step)
	{
		delta |= 2;
		diff -= step;
		vpdiff += step;
	}
	
	step >>= 1;			
	if(diff >= (signed)step)
	{
		delta |= 1;
		vpdiff += step;
	}
	
	if(sign)
		valpred -= vpdiff;
	else	
		valpred += vpdiff;
	
	if(valpred > 32767) 
		valpred = 32767;
	else if (valpred < -32768) 
		valpred = -32768;
	
	delta |= sign;
	index += indexTable[delta];
	if(index < 0) index = 0;
	if(index > 88) index = 88;
	step = stepsizeTable[index];
	
	len_value = len_value - 1;
	
	if (bufferstep)
		outputbuffer = (delta << 4) & 0xf0;
	else
	{
		c = (delta & 0x0f) | outputbuffer;
		ioDataStruct.outdata = c;
		ioDataStruct.outdata_enable = true;
	}
	bufferstep= !bufferstep;
	
	if (!bufferstep)
	{
		ioDataStruct.outdata = outputbuffer;
		ioDataStruct.outdata_enable = true;
	}
}

void adpcm_UT::reset()
{
	state = RESET;
}

void adpcm_UT::run()
{
	reset();

	while(true)
	{
		adpcm_coder();
	}	
}
