//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//-- This confidential and proprietary software may be used only as authorized 
//-- by a licensing agreement from ESD University of Verona - EDALab s.r.l.                      
//--                                                                            
//--                                                                           
//-- Title:             Multimedia System Design Developement
//-- Project:           V-CLIP 1.0
//--    
//-- 
//-- File:              main_RTL.cpp
//-- 
//-- Mod History: $Log: $
//                                                                            
//----------------------------------------------------------------------------
//-- RCS Version:  $Id: $                                           
//----------------------------------------------------------------------------

#include <systemc.h>
#include "adpcm_RTL.h"

/////////////////////////////////////
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	1
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) 
{
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
		fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = atoi(subline.c_str());
				//std::cout << "i-" << test_data_in[c][i] << " " << atoi(subline.c_str());
			}
			c++;
			//std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = atoi(subline.c_str());
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}

////////////////////////////////////////////////
SC_MODULE(testt)
{
	adpcm_RTL ins;
	sc_signal<sc_uint<16> > len;
	sc_signal<short > indata;
	sc_in_clk clk;
	sc_signal<bool > reset;
	
	sc_signal<char > outdata;
	sc_signal<bool > outdata_enable;
	sc_signal<bool > indata_enable;
	sc_signal<bool > len_enable;
	
	void test()
	{
		std::cout << "start simulation " << std::endl;
		
		int c = 0;
		while(true)
		{
			len.write(test_data_in[c][0]);
			indata.write(test_data_in[c][1]);
			reset.write(test_data_in[c][2]);

			std::cout << sc_time_stamp() << "i-" 
				<< len.read() << " "
				<< indata.read() << " "
				<< reset.read() << " "
				<< std::endl;
			
			wait(clk.negedge_event());
			
			std::cout << sc_time_stamp() << "o-" 
				<< outdata.read() << " " 
				<< outdata_enable.read() << " "
				<< indata_enable.read() << " " 
				<< len_enable.read() << " "
				<< std::endl;
			
			if(test_data_out_x[c][0] == 0)
				assert((unsigned)(outdata.read()) == test_data_out[c][0]);
			if(test_data_out_x[c][1] == 0)
				assert((unsigned)(outdata_enable.read()) == test_data_out[c][1]);
			if(test_data_out_x[c][2] == 0)
				assert((unsigned)(indata_enable.read()) == test_data_out[c][2]);
			if(test_data_out_x[c][3] == 0)
				assert((unsigned)(len_enable.read()) == test_data_out[c][3]);
			c++;
		}	
		std::cout << "start simulation " << std::endl;
	}
	
	SC_CTOR(testt):ins("ins")
	{
		//SC_CTHREAD(test, clk_i.pos());
		SC_THREAD(test);
		
		ins.clk(clk);
		ins.len(len);
		ins.indata(indata);
		ins.reset(reset);
		ins.outdata(outdata);
		ins.outdata_enable(outdata_enable);
		ins.indata_enable(indata_enable);
		ins.len_enable(len_enable);
	}
};

///////////////////////////////////////
int sc_main(int argc, char *argv[])
{
	read_test_input();
	
	// start simulation
	sc_clock clk;
	testt tester("tester");
	tester.clk(clk);
	sc_start(10,SC_NS);
	return 0;
}


