#include <iostream>
#include <systemc>
#include "directcontrol_L.h"

//--------------------------------------------------------------------
// Machine: drctCntl
//--------------------------------------------------------------------
//----------------------------------
// Next State Logic (combinatorial)
//----------------------------------
void directControl_L::drctCntl_NextState()
{
/*	std::cout << sc_time_stamp() << "come to comb " << CurrState_drctCntl.read() << " "
	<< NextState_drctCntl.read() << ":"
	<< HCTxPortGnt.read() << " "
	<< HCTxPortRdy.read() << " "
	<< directControlEn.read() << " "
	<< directControlLineState.read() << " "
	<< rst.read()
	<< std::endl;*/
	
  NextState_drctCntl.write(CurrState_drctCntl.read());
  // Set default values for outputs and signals
  next_HCTxPortReq.write(HCTxPortReq.read());
  next_HCTxPortWEn.write(HCTxPortWEn.read());
  next_HCTxPortData.write(HCTxPortData.read());
  next_HCTxPortCntl.write(HCTxPortCntl.read());
	
  switch (CurrState_drctCntl.read())
  {
    case START_DC:
// 			std::cout << sc_time_stamp() << "START_DC " << std::endl;
      NextState_drctCntl.write(CHK_DRCT_CNTL);
      break;
    case CHK_DRCT_CNTL:
// 			std::cout << sc_time_stamp() << "CHK_DRCT_CNTL " << directControlEn.read() << std::endl;
      if (directControlEn.read() == true)	
      {
// 				std::cout << "true " << NextState_drctCntl.read() << std::endl; 
        NextState_drctCntl.write(DRCT_CNTL_WAIT_GNT);
        next_HCTxPortReq.write(true);
      }
      else
      {
// 				std::cout << "false" << std::endl; 
        NextState_drctCntl.write(IDLE_WAIT_GNT);
        next_HCTxPortReq.write(true);
      }
      break;
    case DRCT_CNTL_WAIT_GNT:
// 			std::cout << sc_time_stamp() << "DRCT_CNTL_WAIT_GNT " << std::endl;
      if (HCTxPortGnt.read() == true)	
      {
        NextState_drctCntl.write(DRCT_CNTL_WAIT_RDY);
      }
      break;
    case DRCT_CNTL_CHK_LOOP:
// 			std::cout << sc_time_stamp() << "DRCT_CNTL_CHK_LOOP " << std::endl;
      next_HCTxPortWEn.write(false);
      if (directControlEn.read() == false)	
      {
        NextState_drctCntl.write(CHK_DRCT_CNTL);
        next_HCTxPortReq.write(false);
      }
      else
      {
        NextState_drctCntl.write(DRCT_CNTL_WAIT_RDY);
      }
      break;
    case DRCT_CNTL_WAIT_RDY:
// 			std::cout << sc_time_stamp() << "DRCT_CNTL_WAIT_RDY " << std::endl;
      if (HCTxPortRdy.read() == true)	
      {
        NextState_drctCntl.write(DRCT_CNTL_CHK_LOOP);
        next_HCTxPortWEn.write(true);
//         next_HCTxPortData <= {6'b000000, directControlLineState};
        next_HCTxPortData.write((0x00)|(directControlLineState.read()));
        next_HCTxPortCntl.write(TX_DIRECT_CONTROL);
      }
      break;
    case IDLE_FIN:
// 			std::cout << sc_time_stamp() << "IDLE_FIN " << std::endl;
      next_HCTxPortWEn.write(false);
      next_HCTxPortReq.write(false);
      NextState_drctCntl.write(CHK_DRCT_CNTL);
      break;
    case IDLE_WAIT_GNT:
// 			std::cout << sc_time_stamp() << "IDLE_WAIT_GNT " << std::endl;
      if (HCTxPortGnt.read() == true)	
        NextState_drctCntl.write(IDLE_WAIT_RDY);
      break;
    case IDLE_WAIT_RDY:
// 			std::cout << sc_time_stamp() << "IDLE_WAIT_RDY " << std::endl;
      if (HCTxPortRdy.read() == true)	
      {
        NextState_drctCntl.write(IDLE_FIN);
        next_HCTxPortWEn.write(true);
        next_HCTxPortData.write(0x00);
        next_HCTxPortCntl.write(TX_IDLE);
      }
      break;
  }
// 	std::cout << sc_time_stamp() << "come to comb2 " << CurrState_drctCntl.read() << " "
// 	<< NextState_drctCntl.read() << std::endl;
}

//----------------------------------
// Current State Logic (sequential)
//----------------------------------
void directControl_L::drctCntl_CurrentState()
{
// 	std::cout << sc_time_stamp() << "come to seq " << CurrState_drctCntl.read() << " "
// 	<< NextState_drctCntl.read() << " "
// 	<< HCTxPortGnt.read() << " "
// 	<< HCTxPortRdy.read() << " "
// 	<< directControlEn.read() << " "
// 	<< directControlLineState.read() << " "
// 	<< rst.read()
// 	<< std::endl;

  if (rst.read())	
    CurrState_drctCntl.write(START_DC);
  else
    CurrState_drctCntl.write(NextState_drctCntl.read());

/*	std::cout << sc_time_stamp() << "come to seq2 " << CurrState_drctCntl.read() << " "
	<< NextState_drctCntl.read() << std::endl;*/
}

//----------------------------------
// Registered outputs logic
//----------------------------------
void directControl_L::drctCntl_RegOutput()
{
// 	std::cout << sc_time_stamp() << "come to reg " << std::endl;
  if (rst.read())	
  {
    HCTxPortCntl.write(0x00);
    HCTxPortData.write(0x00);
    HCTxPortWEn.write(false);
    HCTxPortReq.write(false);
  }
  else
  {
    HCTxPortCntl.write(next_HCTxPortCntl.read());
    HCTxPortData.write(next_HCTxPortData.read());
    HCTxPortWEn.write(next_HCTxPortWEn.read());
    HCTxPortReq.write(next_HCTxPortReq.read());
  }
}

void directControl_L::show()
{
	std::cout << sc_time_stamp() << "show: " << next_HCTxPortCntl.read() << " "
		<< next_HCTxPortData.read() << " "
		<< next_HCTxPortReq.read() << " "
		<< next_HCTxPortWEn.read() << " "
		<< rst.read() << " "
		<< CurrState_drctCntl.read() << " "
		<< NextState_drctCntl.read() << std::endl;
}	
