#include <systemc>
#include <iostream>
#include "directcontrol_L.h"

#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	5
#define NUM_OUT 4

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

SC_MODULE(testt)
{
  sc_out<bool>	HCTxPortGnt;
  sc_out<bool>	HCTxPortRdy;
  sc_out<bool>	clk;
  sc_out<bool>	directControlEn;
  sc_out<sc_uint<2> >	directControlLineState;
  sc_out<bool>	rst;

  sc_in<sc_uint<8> >	HCTxPortCntl;
  sc_in<sc_uint<8> >	HCTxPortData;
  sc_in<bool>	HCTxPortReq;
  sc_in<bool>	HCTxPortWEn;

	void stimuli()
	{
		std::string fname = "test_input.txt";
		std::string line;
		std::string subline;
		sc_uint<16> val;
		int c, c1;
		
		// initialize test data
		for(int i = 0; i < MAX_ROW; i++)
			for(int j = 0; j < MAX_COL; j++)
			{
				test_data_in[i][j] = 0;
				test_data_out[i][j] = 0;
				test_data_out_x[i][j] = 0;
			}
		
		// read in test data
		fin.open(fname.c_str());	
		assert(fin.is_open());
		
		getline(fin, line);
		c = 0;
		c1 = 0;
		while(true)
		{
			getline(fin, line);
			if(line.size() == 0)	break;
			
			if(line.find("i_x") != std::string::npos)
			{
				line = line.substr(line.find("i") + 3);
				for(int i = 0; i < NUM_IN; i++)
				{
					subline = Trim(line.substr(0, line.find(":")));
					line = line.substr(line.find(":") + 1);
					assert(subline != "x");
					test_data_in[c][i] = atoi(subline.c_str());
					//std::cout << "i-" << test_data_in[c][i] << " " << atoi(subline.c_str());
				}
				c++;
				//std::cout << std::endl;
			}
			else if(line.find("o_x") != std::string::npos)
			{
				line = line.substr(line.find("o") + 3);
				for(int i = 0; i < NUM_OUT; i++)
				{
					subline = Trim(line.substr(0, line.find(":")));
					line = line.substr(line.find(":") + 1);
					if(subline == "x")
					{
						test_data_out_x[c1][i] = 1;
					}
					else
						test_data_out[c1][i] = atoi(subline.c_str());
					// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
				}
				c1++;
				// 			std::cout << std::endl;
			}
			else
			{
				//std::cout << "else: " << line << std::endl;
				//assert(0);
			}
		}
	
		int c2 = 0;
		std::cout << "enter simulation " << std::endl;
		while(true)
		{
			//std::cout << std::endl;
			wait(5, SC_NS);
			//if(c2 >= 0)
			//{
				HCTxPortGnt.write(test_data_in[c2][0]);
				HCTxPortRdy.write(test_data_in[c2][1]);
				directControlEn.write(test_data_in[c2][2]);
				directControlLineState.write(test_data_in[c2][3]);
				rst.write(test_data_in[c2][4]);
			//}
 			std::cout << sc_time_stamp() << "i-" << HCTxPortGnt.read() << " " << HCTxPortRdy.read() << " " 
				<< directControlEn.read() << " " << directControlLineState.read() << " " << rst.read() << std::endl;

// 			std::cout << "input" << std::endl;
 			wait(2, SC_NS);
// 			
 			std::cout << sc_time_stamp() << "o-" << HCTxPortCntl.read() << " " << HCTxPortData.read() << " " 
				<< HCTxPortReq.read() << " " << HCTxPortWEn.read() << std::endl;
			//if(c2 >= 0)
			//{
				if(test_data_out_x[c2][0] == 0)
					assert((unsigned)(HCTxPortCntl.read()) == test_data_out[c2][0]);
				if(test_data_out_x[c2][1] == 0)
					assert((unsigned)(HCTxPortData.read()) == test_data_out[c2][1]);
				if(test_data_out_x[c2][2] == 0)
					assert((unsigned)(HCTxPortReq.read()) == test_data_out[c2][2]);
				if(test_data_out_x[c2][3] == 0)
					assert((unsigned)(HCTxPortWEn.read()) == test_data_out[c2][3]);
			//}
				wait(3, SC_NS);
			c2++;
		}
	}

	void clk_gen()
	{
		while(true) {
			clk.write(false);
			wait(5, SC_NS);
			clk.write(true);
			wait(5, SC_NS);
		}
	}

	SC_CTOR(testt)
	{
		SC_THREAD(stimuli); 
		SC_THREAD(clk_gen); 
	}
};

// here it goes.
int sc_main(int argc, char *argv[])
{
  sc_set_time_resolution(1, SC_NS);

  sc_signal<bool>	HCTxPortGnt;
  sc_signal<bool>	HCTxPortRdy;
  sc_signal<bool>	clk;
  sc_signal<bool>	directControlEn;
  sc_signal<sc_uint<2> >	directControlLineState;
  sc_signal<bool>	rst;

  sc_signal<sc_uint<8> >	HCTxPortCntl;
  sc_signal<sc_uint<8> >	HCTxPortData;
  sc_signal<bool>	HCTxPortReq;
  sc_signal<bool>	HCTxPortWEn;

	directControl_L ins("ins");
	ins.HCTxPortGnt(HCTxPortGnt);
	ins.HCTxPortRdy(HCTxPortRdy);
	ins.clk(clk);
	ins.directControlEn(directControlEn);
	ins.directControlLineState(directControlLineState);
	ins.rst(rst);
	ins.HCTxPortCntl(HCTxPortCntl);
	ins.HCTxPortData(HCTxPortData);
	ins.HCTxPortReq(HCTxPortReq);
	ins.HCTxPortWEn(HCTxPortWEn);

	testt tb("tester");
	tb.HCTxPortGnt(HCTxPortGnt);
	tb.HCTxPortRdy(HCTxPortRdy);
	tb.clk(clk);
	tb.directControlEn(directControlEn);
	tb.directControlLineState(directControlLineState);
	tb.rst(rst);
	tb.HCTxPortCntl(HCTxPortCntl);
	tb.HCTxPortData(HCTxPortData);
	tb.HCTxPortReq(HCTxPortReq);
	tb.HCTxPortWEn(HCTxPortWEn);

  sc_start(100, SC_NS);

  return 0;
}
