/*****************************************************************************

  The following code is derived, directly or indirectly, from the SystemC
  source code Copyright (c) 1996-2006 by all Contributors.
  All Rights reserved.

  The contents of this file are subject to the restrictions and limitations
  set forth in the SystemC Open Source License Version 2.4 (the "License");
  You may not use this file except in compliance with such restrictions and
  limitations. You may obtain instructions on how to receive a copy of the
  License at http://www.systemc.org/. Software distributed by Contributors
  under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
  ANY KIND, either express or implied. See the License for the specific
  language governing rights and limitations under the License.

 *****************************************************************************/

/*****************************************************************************

  main.cpp - This file instantiates all processes and ties them together
             with signals.

  Original Author: Rashmi Goswami, Synopsys, Inc.

 *****************************************************************************/

/*****************************************************************************

  MODIFICATION LOG - modifiers, enter your name, affiliation, date and
  changes you are making here.

      Name, Affiliation, Date:
  Description of Modification:

 *****************************************************************************/

#include "systemc.h"
#include "fft_flpt.h"

SC_MODULE(source) 
{
	sc_in<bool> data_req; 
	sc_out<float >  out_real;       
	sc_out<float >  out_imag; 
	sc_out<bool> data_valid;       
	sc_in_clk CLK;   
	
	SC_CTOR(source)
	{
		SC_CTHREAD(entry, CLK.pos());
	}
	
	//Process Functionality: in member function below
	void entry();
};

int real[64] =
{
	2011, 2011, 2011, 2011, 2011, 2011, 2011, 2011, 2011, 2011, 2011, 2011, 2011,
	2011, 2011, 2011, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 120, 234,
	456, 768, 456, 234, 120, 0, -120, -234, -456, -768, -456, -234, -120, 123, 34,
	-23, 12, 45, 0, 67, 134, -45, 30, 96, 189, 234, 535, 105, 576
};

int imag[64] =
{
	2011, 2011, 2011, 2011, 2011, 2011, 2011, 2011, 2011, 2011, 2011, 2011, 2011,
	2011, 2011, 2011, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 456, 234,
	120, 0, -120, -234, -456, -768, -456, -234, -120, 0, 120, 234, 456, 768, 29,
	34, 85, 206, 336, 593, 823, 422, -12, -103, -599, -734, -339, -188, -23, 22
};

void source::entry()
{ 
	int i = 0;
	data_valid.write(false);
	
	while(true)
	{ 
		do { wait(); } while ( !(data_req == true) );
		
		out_real.write(real[i]);
		out_imag.write(imag[i]);
		i = i < 63? i+1: 0;
		data_valid.write(true);
		do { wait(); } while ( !(data_req == false) );
		data_valid.write(false);
		wait();
	}
}

struct sink: sc_module {
	sc_in<bool>  data_ready; 
	sc_out<bool> data_ack; 
	sc_in< float > in_real; 
	sc_in< float > in_imag; 

	sc_in_clk CLK;
		
	SC_CTOR(sink)
	{
		SC_CTHREAD(entry, CLK.pos());
	}
	void entry(); 
};

void sink::entry()
{
	sc_int<16> tmp;
	int tmp_out;
	
	data_ack.write(false);
	
	while(true)
	{ 
		do { wait(); } while ( !(data_ready == true) );
		tmp = in_real.read();
		tmp_out = tmp;
		cout <<"real " << tmp_out << "\n";
		tmp = in_imag.read();
		tmp_out = tmp;
		cout <<"imag " << tmp_out << "\n";
		data_ack.write(true);
		do { wait(); } while ( !(data_ready == false) );
		data_ack.write(false); 
	}
}

int sc_main(int ac, char* av[])
{
	sc_signal<float> in_real;
	sc_signal<float> in_imag;
	sc_signal<bool> data_valid;
	sc_signal<bool> data_ack;
	sc_signal<float> out_real;
	sc_signal<float> out_imag;
	sc_signal<bool> data_req;
	sc_signal<bool> data_ready;

	sc_clock clock("CLOCK", 10, SC_NS, 0.5, 0.0, SC_NS);

	fft FFT1("FFTPROCESS"); 
	FFT1.in_real(in_real);
	FFT1.in_imag(in_imag);
	FFT1.data_valid(data_valid);
	FFT1.data_ack(data_ack);
	FFT1.out_real(out_real);
	FFT1.out_imag(out_imag);
	FFT1.data_req(data_req);
	FFT1.data_ready(data_ready);
	FFT1.CLK(clock);

	source SOURCE1("SOURCEPROCESS");
	SOURCE1.data_req(data_req);
	SOURCE1.out_real(in_real);
	SOURCE1.out_imag(in_imag);
	SOURCE1.data_valid(data_valid);
	SOURCE1.CLK(clock);

	sink SINK1("SINKPROCESS");
	SINK1.data_ready(data_ready);
	SINK1.data_ack(data_ack);
	SINK1.in_real(out_real);
	SINK1.in_imag(out_imag);
	SINK1.CLK(clock);

	sc_start(5000, SC_NS);
	return 0;
}
