//////////////////////////////////////////////////////////////////////
////                                                              ////
////  Key generation module header                                ////
////                                                              ////
////  This file is part of the SystemC DES                        ////
////                                                              ////
////  Description:                                                ////
////  Generate a key from the previous one                        ////
////                                                              ////
////  To Do:                                                      ////
////   - nothing                                                  ////
////                                                              ////
////  Author(s):                                                  ////
////      - Javier Castillo, jcastilo@opencores.org               ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2000 Authors and OPENCORES.ORG                 ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE.  See the GNU Lesser General Public License for more ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from http://www.opencores.org/lgpl.shtml                     ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
// CVS Revision History
//
// Format revision  2009-9-3	nannan
//
// $Log: key_gen_H.h,v $
// Revision 1.1  2007-10-23 16:21:01  blancn
// *** empty log message ***
//
// Revision 1.2  2004/08/30 16:55:54  jcastillo
// Used indent command on C code
//
// Revision 1.1.1.1  2004/07/05 17:31:17  jcastillo
// First import
//

#include "systemc.h"

SC_MODULE (key_gen_H)
{
	sc_uint < 56 > previous_key;
	sc_uint < 4 > iteration;
	bool decrypt;	//When decrypting we rotate rigth instead of left
	sc_uint < 56 > non_perm_key;
	sc_uint < 48 > new_key;
	
	void generate_key ();
	unsigned int PC2(unsigned int x, unsigned int y);
	void key_gen_0();
	void key_gen_1();
	void key_gen_2();
	void run ();
	
	SC_CTOR (key_gen_H)
	{
		SC_THREAD (run);
	}
	
	private:
		bool prev0, prev1;
		sc_uint < 56 > prev_key_var, non_perm_key_var;
		sc_uint < 48 > new_key_var;
		sc_uint < 28 > semi_key;
};

unsigned int key_gen_H::PC2 (unsigned int x, unsigned int y)
{
	static unsigned int PC2_Permutation[6][8] = {
		{24, 27, 20, 6, 14, 10, 3, 22},
		{0, 17, 7, 12, 8, 23, 11, 5},
		{16, 26, 1, 9, 19, 25, 4, 15},
		{54, 43, 36, 29, 49, 40, 48, 30},
		{52, 44, 37, 33, 46, 35, 50, 41},
		{28, 53, 51, 55, 32, 45, 39, 42},
	};
	return PC2_Permutation[x][y];
}

void key_gen_H::key_gen_0 ()
{
	if(!decrypt)
	{
		prev0 = semi_key[27];
		semi_key = semi_key << 1;
		semi_key[0] = prev0;
	}
	non_perm_key_var.range (55, 28) = semi_key;
	semi_key = prev_key_var.range (27, 0);
	if(!decrypt)
	{
		prev0 = semi_key[27];
		semi_key = semi_key << 1;
		semi_key[0] = prev0;
	}
}

void key_gen_H::key_gen_1 ()
{	
	if(!decrypt)
	{
		prev0 = semi_key[27];
		semi_key = semi_key << 1;
		semi_key[0] = prev0;
	}
	else
	{
		prev0 = semi_key[0];
		semi_key = semi_key >> 1;
		semi_key[27] = prev0;
	}
	non_perm_key_var.range (55, 28) = semi_key;
	semi_key = prev_key_var.range (27, 0);
	if(!decrypt)
	{
		prev0 = semi_key[27];
		semi_key = semi_key << 1;
		semi_key[0] = prev0;
	}
	else
	{
		prev0 = semi_key[0];
		semi_key = semi_key >> 1;
		semi_key[27] = prev0;
	}
}

void key_gen_H::key_gen_2 ()
{	
	if(!decrypt)
	{
		prev0 = semi_key[27];
		prev1 = semi_key[26];
		semi_key = semi_key << 2;
		semi_key[1] = prev0;
		semi_key[0] = prev1;
	}
	else
	{
		prev0 = semi_key[0];
		prev1 = semi_key[1];
		semi_key = semi_key >> 2;
		semi_key[26] = prev0;
		semi_key[27] = prev1;
	}
	non_perm_key_var.range (55, 28) = semi_key;
	semi_key = prev_key_var.range (27, 0);
	if(!decrypt)
	{
		prev0 = semi_key[27];
		prev1 = semi_key[26];
		semi_key = semi_key << 2;
		semi_key[1] = prev0;
		semi_key[0] = prev1;
	}
	else
	{
		prev0 = semi_key[0];
		prev1 = semi_key[1];
		semi_key = semi_key >> 2;
		semi_key[26] = prev0;
		semi_key[27] = prev1;
	}
}

void key_gen_H::generate_key ()
{
	prev_key_var = previous_key;
	new_key_var = 0;
	new_key = 0;
	non_perm_key_var = 0;
	non_perm_key = 0;
	
	semi_key = prev_key_var.range (55, 28);
	if(iteration == 0)
		key_gen_0();
	else if((iteration == 1) || (iteration == 8) || (iteration == 15))
		key_gen_1();
	else 		
		key_gen_2();
	non_perm_key_var.range (27, 0) = semi_key;
	
	non_perm_key = non_perm_key_var;
	
	//Apply PC_2 permutation
	//Expand the data    
	for(unsigned int i = 0; i < 6; i++)
	{
		for(unsigned int j = 0; j < 8; j++)
			new_key_var[i * 8 + j] = non_perm_key_var[PC2(i, j)]; // fix bug
	}
	
	new_key = new_key_var;
}

void key_gen_H::run()
{
	while(true)
	{
		generate_key();
	}
}
