#include <string>
#include <list>
#include <systemc.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include "key_gen_L.h"

#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	6
#define NUM_OUT 7

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

SC_MODULE(testt)
{
	sc_in<bool > clk;

	key_gen_L ins;
	sc_signal<sc_uint<56> > previous_key;
	sc_signal<sc_uint <4> > iteration;
	sc_signal<bool > decrypt;

	sc_signal<sc_uint<56> > non_perm_key;
	sc_signal<sc_uint<48> > new_key;
	
	sc_uint<56> previous_key_reg;
	sc_uint<4> iteration_reg;
		
	void test()
	{
		int c = 0;
		while(true)
		{
			previous_key_reg.range(55,48) = test_data_in[c][0];
			previous_key_reg.range(47,32) = test_data_in[c][1];
			previous_key_reg.range(31,16) = test_data_in[c][2];
			previous_key_reg.range(15,0) = test_data_in[c][3];
			previous_key.write(previous_key_reg);
			iteration = test_data_in[c][4];
			decrypt = test_data_in[c][5];
			
			wait();
			
			assert(((unsigned) (non_perm_key.read()).range(55,48)) == test_data_out[c][0]);
			assert(((unsigned) (non_perm_key.read()).range(47,32)) == test_data_out[c][1]);
			assert(((unsigned) (non_perm_key.read()).range(31,16)) == test_data_out[c][2]);
			assert(((unsigned) (non_perm_key.read()).range(15,0)) == test_data_out[c][3]);
			assert(((unsigned) (new_key.read()).range(47,32)) == test_data_out[c][4]);
			assert(((unsigned) (new_key.read()).range(31,16)) == test_data_out[c][5]);
			assert(((unsigned) (new_key.read()).range(15,0)) == test_data_out[c][6]);
			c++;
		}
	}

	SC_CTOR(testt): ins("ins")
	{
		std::cout << "start simulation " << std::endl;
		SC_CTHREAD(test, clk.pos());

		ins.previous_key(previous_key);
		ins.iteration(iteration);
		ins.decrypt(decrypt);
		ins.non_perm_key(non_perm_key);
		ins.new_key(new_key);
		
		std::cout << "start simulation " << std::endl;
	}
};

int sc_main(int argc, char* argv[])
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	sc_uint<16> val;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
		}
		
	// read in test data
	fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				test_data_in[c][i] = atoi(subline.c_str());
				//std::cout << "i-" << test_data_in[c][i] << " " << atoi(subline.c_str());
			}
			c++;
			//std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				test_data_out[c1][i] = atoi(subline.c_str());
				//std::cout << "o-" << test_data_in[c][i] << " " << atoi(subline.c_str());
			}
			c1++;
			//std::cout << std::endl;
		}
		else
		{
			std::cout << "else: " << line << std::endl;
			assert(0);
		}
	}
	
	// start simulation
	sc_clock clk;
	testt tester("tester");
	tester.clk(clk);
	sc_start(10,SC_NS);
	return 0;
}
