//////////////////////////////////////////////////////////////////////
////                                                              ////
////  AES moxcolum module implementation                          ////
////                                                              ////
////  This file is part of the SystemC AES                        ////
////                                                              ////
////  Description:                                                ////
////  Mixcolum stage implementation for AES algorithm             ////
////                                                              ////
////  To Do:                                                      ////
////   - done                                                     ////
////                                                              ////
////  Author(s):                                                  ////
////      - Javier Castillo, jcastilo@opencores.org               ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2000 Authors and OPENCORES.ORG                 ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE.  See the GNU Lesser General Public License for more ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from http://www.opencores.org/lgpl.shtml                     ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
// CVS Revision History
//
// Format revision  2009-9-3	nannan
//
// $Log: mixcolum_H.h,v $
// Revision 1.1  2007-10-11 20:37:58  blancn
// *** empty log message ***
//
// Revision 1.1  2005/02/14 11:18:31  jcastillo
// Moved
//
// Revision 1.2  2004/08/30 14:44:44  jcastillo
// Code Formater used to give better appearance to SystemC code
//
// Revision 1.1.1.1  2004/07/05 09:46:22  jcastillo
// First import
//

#include "systemc.h"
#include "word_mixcolum_H.h"

SC_MODULE(mixcolum_H)
{
  bool clk;
  bool reset;
  bool decrypt_i;
  bool start_i;
  sc_biguint<128>  data_i;

  bool ready_o;
  sc_biguint<128> data_o;

  void update();
  void run();
  void mixcol();
  void registers();
  void mux();
  void assign_data_o();

  sc_biguint<128> data_reg, next_data_reg, data_o_reg, next_data_o;
  bool next_ready_o;
  sc_uint<2> state, next_state;
  sc_uint<32> outx, outy, mix_word, outmux;

  word_mixcolum_H *w1;

  SC_CTOR(mixcolum_H)
  {
    SC_THREAD(run);
  }
};

void mixcolum_H::update()
{
  w1->in = mix_word;
  outx = w1->outx;
  outy = w1->outy;
}

void mixcolum_H::run()
{
  w1 = new word_mixcolum_H("w1");

  while(true)
  {
    update();
    assign_data_o();
    mux();
    registers();
    mixcol();
  }
}

void mixcolum_H::mux()
{
  outmux = decrypt_i ? outy : outx;
}

void mixcolum_H::mixcol()
{
  sc_biguint<128> data_i_var;
  sc_uint<32> aux;
  sc_biguint<128> data_reg_var;

  data_i_var = data_i;
  data_reg_var = data_reg;
  next_data_reg = data_reg;
  next_state = state;

  mix_word = 0;

  next_ready_o = 0;
  next_data_o = data_o_reg;

  switch (state)
  {
    case 0:
      if (start_i)
      {
				aux = data_i_var.range(127, 96);
				mix_word = aux;
				data_reg_var.range(127, 96) = outmux;
				next_data_reg = data_reg_var;
				next_state = 1;
      }
      break;
    case 1:
      aux = data_i_var.range(95, 64);
      mix_word = aux;
      data_reg_var.range(95, 64) = outmux;
      next_data_reg = data_reg_var;
      next_state = 2;
      break;
    case 2:
      aux = data_i_var.range(63, 32);
      mix_word = aux;
      data_reg_var.range(63, 32) = outmux;
      next_data_reg = data_reg_var;
      next_state = 3;
      break;
    case 3:
      aux = data_i_var.range(31, 0);
      mix_word = aux;
      data_reg_var.range(31, 0) = outmux;
      next_data_o = data_reg_var;
      next_ready_o = 1;
      next_state = 0;
      break;
    default:
      break;
  }
}

void mixcolum_H::registers()
{
  if (!reset)
  {
    data_reg = 0;
    state = 0;
    ready_o = 0;
    data_o_reg = 0;
  }
  else
  {
    data_reg = next_data_reg; 
    state = next_state; 
    ready_o = next_ready_o; 
    data_o_reg = next_data_o; 
  }
}

void mixcolum_H::assign_data_o()
{
  data_o = data_o_reg;
}


