/*****************************************************************************

  The following code is derived, directly or indirectly, from the SystemC
  source code Copyright (c) 1996-2006 by all Contributors.
  All Rights reserved.

  The contents of this file are subject to the restrictions and limitations
  set forth in the SystemC Open Source License Version 2.4 (the "License");
  You may not use this file except in compliance with such restrictions and
  limitations. You may obtain instructions on how to receive a copy of the
  License at http://www.systemc.org/. Software distributed by Contributors
  under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
  ANY KIND, either express or implied. See the License for the specific
  language governing rights and limitations under the License.

 *****************************************************************************/

/*****************************************************************************
 
  control.cpp  -- Instruction Decode Unit.
 
  Original Author: Martin Wang, Synopsys, Inc.
 
 *****************************************************************************/
 
/*****************************************************************************
 
  MODIFICATION LOG - modifiers, enter your name, affiliation, date and
  changes you are making here.
 
      Name, Affiliation, Date: Nicolas Blanc, ETHZ, 2009
  Description of Modification: many code updates

      Name, Affiliation, Date:
  Description of Modification:
 
 *****************************************************************************/
 

#include "systemc.h"
#include "control_H.h"

//--------------------------------
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	8
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
	fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "i-" << test_data_in[c][i] << " ";
			}
			c++;
			// 			std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}
//--------------------------------

void control_H::run_control()
{
	static sc_uint<32> 	instr_tmp = 0;
	static sc_uint<8> 	 	opcode_tmp = 0;
	static sc_uint<4>   	regA_tmp = 0;
	static sc_uint<4>   	regB_tmp = 0;
	static sc_uint<4>   	regC_tmp = 0;
	static sc_int<32>  	imm_tmp = 0;
	static sc_uint<12>  	offset_tmp = 0;
	static sc_int<32> 		label_tmp = 0;
	static sc_uint<32> 	longlabel_tmp = 0;
	
	static sc_int<32>  lastreg_tmp = 0;
	static sc_int<32>	srcA_tmp = 0;
	static sc_int<32>	srcB_tmp = 0;
	static sc_int<32>	srcC_tmp = 0;
	
	next_pc = true;
	alu_cs = false;
	fpu_cs = false;
	mmx_cs = false;
	dram_cs = false;
	if(instruction_valid == false) 
		return;
	next_pc = false;
	branch_valid = false;
	
	instr_tmp  = instruction;
	opcode_tmp = instr_tmp.range(31,24);
	regC_tmp   = instr_tmp.range(23,20);  
	regA_tmp   = instr_tmp.range(19,16);
	regB_tmp   = instr_tmp.range(15,12);
	imm_tmp    = sc_int<32>(sc_uint<32>(instr_tmp.range(15,0)));
	offset_tmp = instr_tmp.range(11,0);
	label_tmp  = sc_int<16>(instr_tmp.range(15,0));
	longlabel_tmp = instr_tmp.range(23,0);
	
	srcA_tmp = 0xdeadbeef;
	srcB_tmp = 0xdeadbeef;
	srcC_tmp = 0xdeadbeef;
	
	std::cout << pc << "\t| ";
	
	// get values form RFU
	switch(opcode_tmp)
	{
		case 0x4e:	// sw R1, R2, offset
		case 0x10:	// beq R1, R2, label
		case 0x11:	// bne R1, R2, label
		case 0x12:	// bgt R1, R2, label
		case 0x13:	// bge R1, R2, label
		case 0x14:	// blt R1, R2, label
		case 0x15:	// ble R1, R2, label
			rfu_cs = true; 
			rfu_re = true;
			rfu_addr1 = regC_tmp;
			rfu_addr2 = regA_tmp;
			rfu_cs = false;
			rfu_re = false;
			srcC_tmp = rfu_data1;
			srcA_tmp = rfu_data2;
			break;
		case 0x18:	// jr
			rfu_cs = true;
			rfu_re = true;
			rfu_addr1 = 15;
			rfu_cs = false;
			rfu_re = false;
			srcC_tmp = srcA_tmp = srcB_tmp = rfu_data1;
			break;
		default:    // Other cases
			rfu_cs = true;
			rfu_re = true;
			rfu_addr1 = regA_tmp;
			rfu_addr2 = regB_tmp;
			rfu_cs = false;
			rfu_re = false;
			srcA_tmp = rfu_data1;
			srcB_tmp = rfu_data2;
	}
	
	// Execute operation
	switch(opcode_tmp) {
		case 0x0:	// halt
			cout << "halt\n";
			break;
		case 0x01:	// add R1, R2, R3       
			cout << "add R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp << "), R" 
			<< regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 3;	
			alu_cs = true;
			do{} while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0x02:	// addi R1, R2, #value
			cout << "addi R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp  << "), " << imm_tmp <<  "\n";
			src_A = srcA_tmp;
			src_B = imm_tmp;
			alu_op = 3;	
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0x03:	// addc R1, R2, R3 + Carrybit
			cout << "addc R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")+C\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 1;
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0x04:	// sub R1, R2, R3
			cout << "sub R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 4;
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0x05:	// subi R1, R2,  #value
			cout << "subi R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp  << "), " << imm_tmp <<  "\n";
			src_A = srcA_tmp;
			src_B = imm_tmp;
			alu_op = 4;
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0x06:	// subc R1, R2, R3 - Carrybit
			cout << "subc R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")-C\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 2;
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			
			break;
		case 0x07:	// mul R1, R2, R3
			cout << "mul R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 5;
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0x08:	// div R1, R2, R3
			cout << "div R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 6;
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;        
			break;
		case 0x09:	// nand R1, R2, R3 
			cout << "nand R" << regC_tmp << ", R" << regA_tmp << "(="  << srcA_tmp
			<< "), R" << regB_tmp << "(="  << srcB_tmp  << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 8;
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0x0b:	// or R1, R2, R3 
			cout << "or R" << regC_tmp << ", R" << regA_tmp << "(="  << srcA_tmp 
			<< "), R" << regB_tmp << "(="  << srcB_tmp  << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 9;
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0x0c:	// xor R1, R2, R3 
			cout << "xor R" << regC_tmp << ", R" << regA_tmp << "(="  << srcA_tmp 
			<< "), R" << regB_tmp << "(="  << srcB_tmp  << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 10;
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0x0d:      // NOT R1, R2
			cout << "not R" <<  regC_tmp << ", R" <<  regA_tmp << "(="  << srcA_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = 0;
			alu_op = 11;
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0x0e:      // modulo R1 = R2 mod R3
			cout << "modulo R" << regC_tmp << ", R" << regA_tmp << "(="  << srcA_tmp 
			<< "), R" << regB_tmp << "(="  << srcB_tmp  << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 14;
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0x4d:	// lw R1, R2, offset
			cout << "lw R"<<  regC_tmp << ", R" <<  regA_tmp <<"(="  <<  srcA_tmp  << "), "
			<<  offset_tmp  << "\n";
			dram_cs = true;
			dram_we = false;
			offset_tmp = offset_tmp + srcA_tmp;
			dram_addr = offset_tmp;
			do{}while(dram_rd_valid == false);
			dram_cs = false;
			srcC_tmp = dram_dout;     
			break;
		case 0x4e:	// sw R1, R2, offset
			cout << "sw R"<<  regC_tmp << "(=" << srcC_tmp  <<"), R" 
			<< regA_tmp <<"(="  <<  srcA_tmp  << "), "
			<<  offset_tmp  << "\n";
			dram_cs = true;
			dram_we = true;
			offset_tmp = offset_tmp + srcA_tmp;
			dram_addr = offset_tmp;
			dram_datain = srcC_tmp;
			dram_cs = false;
			dram_we = false;
			break;
		case 0x0f:	// mov R1, R2
			cout << "mov R" <<  regC_tmp << ", R" <<  regA_tmp << "(=" <<  srcA_tmp <<")\n";
			src_A = srcA_tmp;
			src_B = 0;
			alu_op = 3;	
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0x10:	// beq R1, R2, label
			cout << "beq R" <<  regC_tmp << "(="<< srcC_tmp <<"), R" << regA_tmp << "(=" <<  srcA_tmp << "), " <<
			label_tmp  << "\n";
			if (srcC_tmp == srcA_tmp) {
				branch_target_address = pc + sc_int<32>(label_tmp);
				branch_valid = true;
			}
			break;
		case 0x11:	// bne R1, R2, label
			cout << "bne R" <<  regC_tmp << "(="<< srcC_tmp <<"), R" << regA_tmp << "(=" <<  srcA_tmp << "), " <<
			label_tmp  << "\n";
			if (srcC_tmp != srcA_tmp) {
				branch_target_address = pc + sc_int<32>(label_tmp);
				branch_valid = true;
			}
			break;
		case 0x12:	// bgt R1, R2, label
			cout << "bgt R" <<  regC_tmp << "(="<< srcC_tmp <<"), R" << regA_tmp << "(=" <<  srcA_tmp << "), " <<
			label_tmp  << "\n";
			if (srcC_tmp > srcA_tmp) {
				branch_target_address = pc + sc_int<32>(label_tmp);
				branch_valid = true;
			}
			break;
		case 0x13:	// bge R1, R2, label
			cout << "bge R" <<  regC_tmp << "(="<< srcC_tmp <<"), R" << regA_tmp << "(=" <<  srcA_tmp << "), " <<
			label_tmp  << "\n";
			if (srcC_tmp >= srcA_tmp) {
				branch_target_address = pc + sc_int<32>(label_tmp);
				branch_valid = true;
			}
			break;
		case 0x14:	// blt R1, R2, label
			cout << "blt R" <<  regC_tmp << "(="<< srcC_tmp <<"), R" << regA_tmp << "(=" <<  srcA_tmp << "), " <<
			label_tmp  << "\n";
			if (srcC_tmp < srcA_tmp) {
				branch_target_address = pc + sc_int<32>(label_tmp);
				branch_valid = true;
			}
			break;
		case 0x15:	// ble R1, R2, label
			cout << "ble R" <<  regC_tmp << "(="<< srcC_tmp <<"), R" << regA_tmp << "(=" <<  srcA_tmp << "), " <<
			label_tmp  << "\n";
			if (srcC_tmp < srcA_tmp) {
				branch_target_address = pc + sc_int<32>(label_tmp);
				branch_valid = true;
			}
			break;
		case 0x16:	// j label
			cout <<"j "  <<  longlabel_tmp  << "\n";
			branch_target_address = longlabel_tmp;
			branch_valid = true;
			break;
		case 0x17:	// jal label for procedure call
			cout <<"jal "  <<  longlabel_tmp  << "\n";
			branch_target_address = longlabel_tmp;
			branch_valid = true;
			break;
		case 0x18:	// jr
			cout << "jr\n";
			branch_target_address = sc_uint<32>(srcC_tmp);
			branch_valid = true;
			break;
		case 0x29:      // fadd R1, R2, R3
			cout << "fadd R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 3;
			fpu_cs = true;
			do{} while(fpu_valid == false);
			fpu_cs = false;
			srcC_tmp = fpu_dout;
			break;
		case 0x2a:      // fsub R1, R2, R3
			cout << "fsub R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 4;
			fpu_cs = true;
			do {  } while(fpu_valid == false);
			fpu_cs = false;
			srcC_tmp = fpu_dout;
			break;
		case 0x2b:      // fmul R1, R2, R3
			cout << "fmul R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 5;
			fpu_cs = true;
			do {  } while(fpu_valid == false);
			fpu_cs = false;
			srcC_tmp = fpu_dout;
			break;
		case 0x2c:      // fdiv R1, R2, R3
			cout << "fdiv R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 6;
			fpu_cs = true;
			do {  } while(fpu_valid == false);
			fpu_cs = false;
			srcC_tmp = fpu_dout;
			break;
		case 0x31:      // mmxadd R1, R2, R3
			cout << "mmxadd R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 3;
			mmx_cs = true;
			do {  } while(mmx_valid == false);
			mmx_cs = false;
			srcC_tmp = mmx_dout;     
			break;
		case 0x32:      // mmxadds R1, R2, R3
			cout << "mmxadds R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 4;
			mmx_cs = true;
			do {  } while(mmx_valid == false);
			mmx_cs = false;
			srcC_tmp = mmx_dout;
			break;
		case 0x33:      // mmxsub R1, R2, R3
			cout << "mmxsub R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 5;
			mmx_cs = true;
			do{} while(mmx_valid == false);
			mmx_cs = false;
			srcC_tmp = mmx_dout;
			break;
		case 0x34:      // mmxsubs R1, R2, R3
			cout << "mmxsubs R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 6;
			mmx_cs = true;
			if(mmx_valid == true)
			{
				mmx_cs = false;
				srcC_tmp = mmx_dout;
			}
			break;
		case 0x35:      // pmadd R1, R2, R3
			cout << "pmadd R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 7;
			mmx_cs = true;
			do{} while(mmx_valid == false);
			mmx_cs = false;
			srcC_tmp = mmx_dout;        
			break;
		case 0x36:      // pack R1, R2, R3
			cout << "pack R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 8;
			mmx_cs = true;
			do{} while(mmx_valid == false);
			mmx_cs = false;
			srcC_tmp = mmx_dout;
			break;
		case 0x37:      // mmxck R1, R2, R3 MMX Chroma Keying
			cout << "mmxck R" << regC_tmp << ", R" << regA_tmp << "(=" << srcA_tmp 
			<< "), R" << regB_tmp << "(=" << srcB_tmp << ")\n";
			src_A = srcA_tmp;
			src_B = srcB_tmp;
			alu_op = 9;
			mmx_cs = true;
			do{} while(mmx_valid == false);
			mmx_cs = false;
			srcC_tmp = mmx_dout;
			break;
		case 0xe0:      // flush register
			cout << "flush\n";
			src_A = 0;
			src_B = 0;
			alu_op = 0;
			rfu_rst = true;
			rfu_rst = false;
			break;
		case 0xf0:      // ldpid process_number
			cout << "ldpid " <<  longlabel_tmp  << "\n";
			pid_data = sc_int<32>(longlabel_tmp);
			pid_valid = false;   
			break;
		case 0xf1:	// movi R1, #value
			cout << "movi R" << regC_tmp << ", " << imm_tmp << "\n"; 
			src_A = imm_tmp;
			src_B = 0;
			alu_op = 3;	
			alu_cs = true;
			do{}while(alu_valid == false);
			alu_cs = false;
			srcC_tmp = alu_dout;
			break;
		case 0xff:      // QUIT
			cout << "quit\n";
			while(true); 
			break;
		default :
			cout << "OPCODE "  << opcode_tmp  << " IS INVALID!\n";
			break;
	}
	
	// Writeback
	switch(opcode_tmp)
	{
		case 0x01:	// add R1, R2, R3       
		case 0x02:	// addi R1, R2, #value
		case 0x03:	// addc R1, R2, R3 + Carrybit
		case 0x04:	// sub R1, R2, R3
		case 0x05:	// subi R1, R2,  #value
		case 0x06:	// subc R1, R2, R3 - Carrybit
		case 0x07:	// mul R1, R2, R3
		case 0x08:	// div R1, R2, R3
		case 0x09:	// nand R1, R2, R3 
		case 0x0b:	// or R1, R2, R3 
		case 0x0c:	// xor R1, R2, R3 
		case 0x0d:  // NOT R1, R2
		case 0x0e:  // modulo R1 = R2 mod R3
		case 0x4d:	// lw R1, R2, offset
		case 0x0f:	// mov R1, R2
		case 0x29:  // fadd R1, R2, R3
		case 0x2a:  // fsub R1, R2, R3
		case 0x2b:  // fmul R1, R2, R3
		case 0x2c:  // fdiv R1, R2, R3
		case 0x31:  // mmxadd R1, R2, R3
		case 0x32:  // mmxadds R1, R2, R3
		case 0x33:  // mmxsub R1, R2, R3
		case 0x34:  // mmxsubs R1, R2, R3
		case 0x35:  // pmadd R1, R2, R3
		case 0x36:  // pack R1, R2, R3
		case 0x37:  // mmxck R1, R2, R3 MMX Chroma Keying
		case 0xf1:	// movi R1, #value
			std::cout << "\t  R"<< regC_tmp << "=" << srcC_tmp << "\n";
			rfu_cs = true;
			rfu_we = true;
			rfu_addr3 = regC_tmp;
			rfu_data3 = srcC_tmp;        
			rfu_cs = false;
			rfu_we = false;        
			break;
		case 0x17:	// jal label for procedure call
			rfu_cs = true;
			rfu_we = true;
			rfu_addr3 = 15;
			rfu_data3 = pc+1;
			rfu_cs = false;
			rfu_we = false;
			break;
		default:
			break;
	}
} // end of run function

void control_H::run()
{	
	branch_valid = false;
	alu_cs = false;
	fpu_cs = false;
	mmx_cs = false;
	
	//---------------------------
	read_test_input();
// 	int c = 0;
	//----------------------------
	
	while (true)
	{
		//----------------------------
		//----------------------------
		
		run_control();	
		
		//----------------------------
		//----------------------------
	}
} // end of run function
