/*****************************************************************************

  The following code is derived, directly or indirectly, from the SystemC
  source code Copyright (c) 1996-2006 by all Contributors.
  All Rights reserved.

  The contents of this file are subject to the restrictions and limitations
  set forth in the SystemC Open Source License Version 2.4 (the "License");
  You may not use this file except in compliance with such restrictions and
  limitations. You may obtain instructions on how to receive a copy of the
  License at http://www.systemc.org/. Software distributed by Contributors
  under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
  ANY KIND, either express or implied. See the License for the specific
  language governing rights and limitations under the License.

 *****************************************************************************/

/*****************************************************************************
 
  exec.cpp -- Integer Execution Unit.
 
  Original Author: Martin Wang, Synopsys, Inc.
 
 *****************************************************************************/
 
/*****************************************************************************
 
  MODIFICATION LOG - modifiers, enter your name, affiliation, date and
  changes you are making here.
 
      Name, Affiliation, Date:
  Description of Modification:
 
 *****************************************************************************/


#include "systemc.h"

struct exec_h : sc_module {
  bool   	       	reset;			// reset not used.
  bool   	       	cs;		      // chip select
  sc_uint<8>    	opcode;			// opcode from ID
  sc_int<32>      dina;			// operand A
  sc_int<32>      dinb;			// operand B
  bool              	C;			// Carry bit 
  bool              	V;			// Overflow bit
  bool              	Z;			// Zero bit
  sc_int<32>         	dout;			// output data
  bool   	       	out_valid;		// output valid

  SC_CTOR(exec_h) {
      SC_THREAD(run);
  }
  
  void run();
};

void exec_h::run()
{
  sc_int<32>			      opcode_tmp = 0;
  sc_int<32>				    add1_tmp = 0;
  sc_int<32>		        dina_tmp = 0;
  sc_int<32>		        dinb_tmp = 0;
  sc_int<32>		        dout_tmp = 0;
  //
  // main loop
  //
  // initialization of output
  while(1) 
	{
    if (cs == true) 
		{
      dina_tmp = dina;
      dinb_tmp = dinb;
      opcode_tmp = opcode;

      // output MUX
      switch (opcode_tmp) 
			{
        case 0:         // Stall 
          dout_tmp = dout_tmp;
          break;
        case 1: 	// add with carry
          dout_tmp = dina_tmp + dinb_tmp + add1_tmp;
          break;
        case 2: 	// sub with carry
          dout_tmp = dina_tmp - dinb_tmp - add1_tmp;
          break;
        case 3:         // add without carry
          dout_tmp = dina_tmp + dinb_tmp;
          break;
        case 4:         // sub without carry
          dout_tmp = dina_tmp - dinb_tmp;
          break;
        case 5:         // multiply assume 2 clock cycle multiplication
          dout_tmp = dina_tmp * dinb_tmp;
          break;
        case 6:         // divide assume 2 clock cycle multiplication
          if (dinb_tmp == 0) 
					{
            std::cout << "Division Exception - Divide by zero \n";
          } 
					else 
					{
            dout_tmp = dina_tmp / dinb_tmp;
          }
          break;
        case 7:         // bitwise NAND
          dout_tmp = ~(dina_tmp & dinb_tmp);
          break;
        case 8:         // bitwise AND 
          dout_tmp = dina_tmp & dinb_tmp;
          break;
        case 9:         // bitwise OR
          dout_tmp = dina_tmp | dinb_tmp;
          break;
        case 10:         // bitwise XOR
          dout_tmp = dina_tmp ^ dinb_tmp;
          break;
        case 11:         // bitwise complement
          dout_tmp = ~ dina_tmp;
          break;
        case 12:         // left shift
          dout_tmp = dina_tmp << dinb_tmp;
          break;
        case 13:         // right shift
          dout_tmp = dina_tmp >> dinb_tmp;
          break;
        case 14: 	// modulo
          dout_tmp = dina_tmp % dinb_tmp;
          break;
        default:
          std::cout << "ALU: Bad Opcode (" << opcode_tmp << ")\n";
          break;
      }

      dout = dout_tmp;
      out_valid = true;

      if (dout_tmp == 0) 
			{
        Z = true;
      } 
			else 
			{
        Z = false;
      }
      if (dout_tmp > 2^32) 
			{
        V = true;
      }
			else 
			{
        V = false;
      }
      out_valid = false;
    } 
		else 
		{}
  }
}

