/*****************************************************************************

  The following code is derived, directly or indirectly, from the SystemC
  source code Copyright (c) 1996-2006 by all Contributors.
  All Rights reserved.

  The contents of this file are subject to the restrictions and limitations
  set forth in the SystemC Open Source License Version 2.4 (the "License");
  You may not use this file except in compliance with such restrictions and
  limitations. You may obtain instructions on how to receive a copy of the
  License at http://www.systemc.org/. Software distributed by Contributors
  under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
  ANY KIND, either express or implied. See the License for the specific
  language governing rights and limitations under the License.

 *****************************************************************************/

/*****************************************************************************
 
  fetch.cpp -- Instruction Fetch Unit.
 
  Original Author: Martin Wang, Synopsys, Inc.
 
 *****************************************************************************/
 
/*****************************************************************************
 
  MODIFICATION LOG - modifiers, enter your name, affiliation, date and
  changes you are making here.
 
      Name, Affiliation, Date:
  Description of Modification:
 
 *****************************************************************************/
 
#include "systemc.h"
#include "fetch_L.h"


#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	5
#define NUM_OUT 4

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

SC_MODULE(testt)
{
	sc_out<sc_uint<32> >	ramdata;
	sc_out<sc_uint<32> >	branch_address;
	sc_out<bool>	next_pc;
	sc_out<bool>	branch_valid;
	sc_out<bool >	interrupt;
	sc_out<sc_uint<32> >	int_vectno;
	sc_out<bool>	ram_valid;
	sc_out<bool>	CLK;
	
	sc_in<bool>	ram_cs;
	sc_in<sc_uint<32> >	address;
	sc_in<sc_uint<32> >	instruction;
	sc_in<bool>	instruction_valid;
	sc_in<sc_uint<32> >	program_counter;
	sc_in<bool>	interrupt_ack;
	sc_in<bool>	branch_clear;
	sc_in<bool>	reset;
	
	void stimuli()
	{
		std::string fname = "test_input.txt";
		std::string line;
		std::string subline;
		int c, c1;
		
		// initialize test data
		for(int i = 0; i < MAX_ROW; i++)
			for(int j = 0; j < MAX_COL; j++)
			{
				test_data_in[i][j] = 0;
				test_data_out[i][j] = 0;
				test_data_out_x[i][j] = 0;
			}
			
			// read in test data
		fin.open(fname.c_str());	
		assert(fin.is_open());
		
		getline(fin, line);
		c = 0;
		c1 = 0;
		while(true)
		{
			getline(fin, line);
			if(line.size() == 0)	break;
			
			if(line.find("i_x") != std::string::npos)
			{
				line = line.substr(line.find("i") + 3);
				for(int i = 0; i < NUM_IN; i++)
				{
					subline = Trim(line.substr(0, line.find(":")));
					line = line.substr(line.find(":") + 1);
					assert(subline != "x");
					test_data_in[c][i] = atoi(subline.c_str());
					//std::cout << "i-" << test_data_in[c][i] << " " << atoi(subline.c_str());
				}
				c++;
				//std::cout << std::endl;
			}
			else if(line.find("o_x") != std::string::npos)
			{
				line = line.substr(line.find("o") + 3);
				for(int i = 0; i < NUM_OUT; i++)
				{
					subline = Trim(line.substr(0, line.find(":")));
					line = line.substr(line.find(":") + 1);
					if(subline == "x")
					{
						test_data_out_x[c1][i] = 1;
					}
					else
						test_data_out[c1][i] = atoi(subline.c_str());
					// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
				}
				c1++;
				// 			std::cout << std::endl;
			}
			else
			{
				//std::cout << "else: " << line << std::endl;
				//assert(0);
			}
		}
		
		int c2 = 0;
		std::cout << "enter simulation " << std::endl;
		while(true)
		{
			std::cout << "enter simulation " << std::endl;
			wait(5, SC_NS);
			//if(c2 >= 0)
			//{
			ramdata.write(test_data_in[c2][0]);
			branch_address.write(test_data_in[c2][1]);
			next_pc.write(test_data_in[c2][2]);
			branch_valid.write(test_data_in[c2][3]);
			interrupt.write(test_data_in[c2][4]);
			int_vectno.write(test_data_in[c2][3]);
			ram_valid.write(test_data_in[c2][4]);
				//}
			std::cout << sc_time_stamp() << "i-" 
				<< ramdata.read() << " " 
				<< branch_address.read() << " " 
				<< next_pc.read() << " " 
				<< branch_valid.read() << " " 
				<< interrupt.read() << " " 
				<< int_vectno.read() << " " 
				<< ram_valid.read() << std::endl;
				
				// 			std::cout << "input" << std::endl;
			wait(5, SC_NS);
				// 			
			std::cout << sc_time_stamp() << "o-" 
				<< ram_cs.read() << " " 
				<< address.read() << " " 
				<< instruction.read() << " " 
				<< instruction_valid.read() << " "
				<< program_counter.read() << " "
				<< interrupt_ack.read() << " "
				<< branch_clear.read() << " "
				<< reset.read() << " "
				<< std::endl;
				//if(c2 >= 0)
				//{
			if(test_data_out_x[c2][0] == 0)
				assert((unsigned)(ram_cs.read()) == test_data_out[c2][0]);
			if(test_data_out_x[c2][1] == 0)
				assert((unsigned)(address.read()) == test_data_out[c2][1]);
			if(test_data_out_x[c2][2] == 0)
				assert((unsigned)(instruction.read()) == test_data_out[c2][2]);
			if(test_data_out_x[c2][3] == 0)
				assert((unsigned)(instruction_valid.read()) == test_data_out[c2][3]);
			if(test_data_out_x[c2][4] == 0)
				assert((unsigned)(program_counter.read()) == test_data_out[c2][4]);
			if(test_data_out_x[c2][5] == 0)
				assert((unsigned)(interrupt_ack.read()) == test_data_out[c2][5]);
			if(test_data_out_x[c2][6] == 0)
				assert((unsigned)(branch_clear.read()) == test_data_out[c2][6]);
			if(test_data_out_x[c2][7] == 0)
				assert((unsigned)(reset.read()) == test_data_out[c2][7]);
			//}
// 			wait(3, SC_NS);
			c2++;
	}
}

void clk_gen()
{
	while(true) {
		CLK.write(false);
		wait(5, SC_NS);
		CLK.write(true);
		wait(5, SC_NS);
	}
}

SC_CTOR(testt)
{
	SC_THREAD(stimuli); 
	SC_THREAD(clk_gen); 
}
};

// here it goes.
int sc_main(int argc, char *argv[])
{
	sc_set_time_resolution(1, SC_NS);
	
	sc_signal<sc_uint<32> > 		ramdata;	// instruction from RAM    
	sc_signal<sc_uint<32> > 		branch_address; // branch target address   
	sc_signal<bool>  			  next_pc;   	// pc ++
	sc_signal<bool>  			  branch_valid;  	// branch_valid
	sc_signal<bool>  			  interrupt;   	// interrrupt 
	sc_signal<sc_uint<32> >  		int_vectno;   	// interrupt vector number
	sc_signal<bool>  			  ram_valid;   	// ram valid
	
	sc_signal<bool>        ram_cs;     // RAM chip select
	sc_signal<sc_uint<32> >   address;      // address send to RAM
	sc_signal<sc_uint<32> >  	instruction; 	 // instruction send to ID
	sc_signal<bool>  		  instruction_valid; // inst valid
	sc_signal<sc_uint<32> >  	program_counter; // program counter	
	sc_signal<bool>  		  interrupt_ack;   // interrupt acknowledge
	sc_signal<bool>  		  branch_clear;    // clear outstanding branch
	sc_signal<bool>  		  reset;  	// reset
	sc_signal<bool>  		  CLK;  	// reset
	
	fetch_L ins("ins");
	ins.ramdata(ramdata);
	ins.branch_address(branch_address);
	ins.next_pc(next_pc);
	ins.branch_valid(branch_valid);
	ins.interrupt(interrupt);
	ins.int_vectno(int_vectno);
	ins.ram_valid(ram_valid);
	ins.ram_cs(ram_cs);
	ins.address(address);
	ins.instruction(instruction);
	ins.instruction_valid(instruction_valid);
	ins.program_counter(program_counter);
	ins.interrupt_ack(interrupt_ack);
	ins.branch_clear(branch_clear);
	ins.reset(reset);
	ins.CLK(CLK);
	
	testt tb("tester");
	tb.ramdata(ramdata);
	tb.branch_address(branch_address);
	tb.next_pc(next_pc);
	tb.branch_valid(branch_valid);
	tb.interrupt(interrupt);
	tb.int_vectno(int_vectno);
	tb.ram_valid(ram_valid);
	tb.ram_cs(ram_cs);
	tb.address(address);
	tb.instruction(instruction);
	tb.instruction_valid(instruction_valid);
	tb.program_counter(program_counter);
	tb.interrupt_ack(interrupt_ack);
	tb.branch_clear(branch_clear);
	tb.reset(reset);
	tb.CLK(CLK);
	
	sc_start(120, SC_NS);
	
	return 0;
}
