module floating(cs, opcode, dina, dinb, dout, out_valid, CLK);
input cs;
input [7:0] opcode;
input [31:0] dina;
input [31:0] dinb;
input CLK;

output [31:0] dout;
output out_valid;

`define STALL	3'b000
`define FADD	3'b001
`define FSUB	3'b010
`define FMUL	3'b011
`define FDIV	3'b100

reg [31:0] opcode_tmp;
reg [31:0] dout_tmp;
reg [31:0] dest_tmp;

reg [31:0] fpua_sign_tmp;
reg [31:0] fpua_exponent_tmp;
reg [31:0] fpua_significand_tmp;
reg [31:0] fpub_sign_tmp;
reg [31:0] fpub_exponent_tmp;
reg [31:0] fpub_significand_tmp;
reg [2:0]  opcode_encode;
reg [31:0] fpua_tmp;
reg [31:0] fpub_tmp;

reg [31:0] exponent_diff_tmp;
reg [31:0] exponent_tmp;
reg [31:0] significant_result;
reg [31:0] overflow_sign_tmp;
reg [31:0] result_exp_tmp;
reg [31:0] result_sign_tmp;

reg [31:0] dout;
reg out_valid;

initial
begin
	opcode_tmp = 0;
	dout_tmp = 0;
	dest_tmp = 0;

	exponent_diff_tmp = 0;
	exponent_tmp = 0;
	significant_result = 0;
	overflow_sign_tmp = 0;
	result_exp_tmp = 0;
	result_sign_tmp = 0;
	#3;
end

always @(posedge opcode_tmp)
begin
	if(cs == 1)
	begin
		opcode_tmp = opcode;
		fpua_tmp = dina;
		fpub_tmp = dinb;

		fpua_sign_tmp = (fpua_tmp & 32'h80000000) >> 31;
		fpub_sign_tmp = (fpub_tmp & 32'h80000000) >> 31;

		fpua_exponent_tmp = (fpua_tmp & 32'h7f800000) >> 23;
		fpub_exponent_tmp = (fpub_tmp & 32'h7f800000) >> 23;

		fpua_significand_tmp = (fpua_tmp & 32'h007fffff);
		fpub_significand_tmp = (fpub_tmp & 32'h007fffff);

		exponent_diff_tmp = fpua_exponent_tmp - fpub_exponent_tmp;

		if(exponent_diff_tmp > 0)
		begin
			exponent_tmp = fpua_exponent_tmp;
			fpub_significand_tmp = fpub_significand_tmp >> exponent_diff_tmp;
			fpub_exponent_tmp = fpua_exponent_tmp;
		end
		else
		begin
			exponent_tmp = fpub_exponent_tmp;
			fpua_significand_tmp = fpua_significand_tmp >> exponent_diff_tmp;
			fpua_exponent_tmp = fpub_exponent_tmp;
		end
		#1;

		case(opcode_tmp)
			0:	//Stall
			begin
				opcode_encode = `STALL;
				dout_tmp = dout_tmp;
				#1;
			end
			3:  //add
			begin
				opcode_encode = `FADD;
				significant_result = fpua_significand_tmp + fpub_significand_tmp;
				#1;
			end
			4:
			begin
				opcode_encode = `FSUB;
				significant_result = fpua_significand_tmp - fpub_significand_tmp;
				#1;
			end
			5:
			begin
				opcode_encode = `FMUL;
				significant_result = fpua_significand_tmp * fpub_significand_tmp;
				fpub_exponent_tmp = fpub_exponent_tmp * 2;
				#1;
			end
			6:
			begin
				opcode_encode = `FDIV;
				significant_result = fpua_significand_tmp / fpub_significand_tmp;
				#1;
			end
			default:
			begin
				#1;
			end
		endcase
		
		overflow_sign_tmp = (significant_result & 32'hff800000) >> 23;
		dout_tmp = (significant_result << overflow_sign_tmp) & 32'h007fffff ;
		result_exp_tmp = fpub_exponent_tmp  + overflow_sign_tmp; 
		dout_tmp = dout_tmp | ((result_exp_tmp << 23) & 32'h7f800000) ;
		result_sign_tmp = fpua_sign_tmp;
		dout_tmp = dout_tmp | ((result_sign_tmp << 31) & 32'h80000000) ;
		dout = dout_tmp;
		out_valid = 1;
		#1;
		out_valid = 0;
		#1;
	end
end
endmodule
