/*****************************************************************************

  The following code is derived, directly or indirectly, from the SystemC
  source code Copyright (c) 1996-2006 by all Contributors.
  All Rights reserved.

  The contents of this file are subject to the restrictions and limitations
  set forth in the SystemC Open Source License Version 2.4 (the "License");
  You may not use this file except in compliance with such restrictions and
  limitations. You may obtain instructions on how to receive a copy of the
  License at http://www.systemc.org/. Software distributed by Contributors
  under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
  ANY KIND, either express or implied. See the License for the specific
  language governing rights and limitations under the License.

 *****************************************************************************/

/*****************************************************************************
 
  floating.cpp -- Floating Point Execution Unit.
 
  Original Author: Martin Wang, Synopsys, Inc.
 
 *****************************************************************************/
 
/*****************************************************************************
 
  MODIFICATION LOG - modifiers, enter your name, affiliation, date and
  changes you are making here.
 
      Name, Affiliation, Date: Nicolas Blanc, ETHZ, 2009
  Description of Modification: many code updates

      Name, Affiliation, Date:
  Description of Modification:
 
 *****************************************************************************/
 

#include "systemc.h"
#include "floating_H.h"

//--------------------------------
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	8
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
	fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "i-" << test_data_in[c][i] << " ";
			}
			c++;
			// 			std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}
//--------------------------------

void floating_H::run_floating()
{
  static sc_int<32>	opcode_tmp = 0;
	static sc_int<32>	dout_tmp = 0;
	static sc_uint<32>	dest_tmp = 0;

	static sc_uint<32>	fpua_sign_tmp;
	static sc_uint<32>	fpua_exponent_tmp;
	static sc_uint<32>	fpua_significand_tmp;
	static sc_uint<32>	fpub_sign_tmp;
	static sc_uint<32>	fpub_exponent_tmp;
	static sc_uint<32>	fpub_significand_tmp;
  const char *opcode_encode="";
	static sc_uint<32>	fpua_tmp;
	static sc_uint<32>	fpub_tmp;

	static sc_int<32>	exponent_diff_tmp = 0;
	static sc_int<32>	exponent_tmp = 0;
	static sc_uint<32>	significant_result = 0;
	static sc_uint<32>	overflow_sign_tmp = 0;
	static sc_uint<32>	result_exp_tmp = 0;
	static sc_uint<32>	result_sign_tmp = 0;

	if(cs == true) 
		return;
	
	opcode_tmp = opcode;
	fpua_tmp = dina;
	fpub_tmp = dinb;

	fpua_sign_tmp = (fpua_tmp & 0x80000000) >> 31 ;
	fpub_sign_tmp = (fpub_tmp & 0x80000000) >> 31 ;

	fpua_exponent_tmp = (fpua_tmp & 0x7f800000) >> 23 ;
	fpub_exponent_tmp = (fpub_tmp & 0x7f800000) >> 23 ;

	fpua_significand_tmp = (fpua_tmp & 0x007fffff) ;
	fpub_significand_tmp = (fpub_tmp & 0x007fffff) ;

	exponent_diff_tmp = sc_int<32>(fpua_exponent_tmp) - sc_int<32>(fpub_exponent_tmp);

	if (exponent_diff_tmp > 0) 
	{
		exponent_tmp = fpua_exponent_tmp;
	//printf("shift significant B to Right\n");
		fpub_significand_tmp = fpub_significand_tmp >> exponent_diff_tmp ;
		fpub_exponent_tmp = fpua_exponent_tmp;
	} 
	else 
	{
		exponent_tmp = fpub_exponent_tmp;
	//printf("shift significant A to Right\n");
		fpua_significand_tmp = fpua_significand_tmp >> exponent_diff_tmp ;
		fpua_exponent_tmp = fpub_exponent_tmp;
	}

	// output MUX
	switch (opcode_tmp) 
	{
		case 0:         // Stall
			opcode_encode = "STALL";
			dout_tmp = dout_tmp;
			break;
		case 3:         // add
			opcode_encode = "FADD";
			significant_result = sc_int<32>(fpua_significand_tmp) + sc_int<32>(fpub_significand_tmp); 
			break;
		case 4:         // sub
			opcode_encode = "FSUB";
			significant_result = sc_int<32>(fpua_significand_tmp) - sc_int<32>(fpub_significand_tmp); 
			break;
		case 5:         // mul
			opcode_encode = "FMUL";
			significant_result = sc_int<32>(fpua_significand_tmp) * sc_int<32>(fpub_significand_tmp); 
			fpub_exponent_tmp *= 2;			// exponent is doubled in value
			break;
		case 6:         // div
			opcode_encode = "FDIV";
			significant_result = sc_int<32>(fpua_significand_tmp) / sc_int<32>(fpub_significand_tmp); 
			break;
		default:
			cout << "FPU: Bad Opcode " << opcode_tmp << "\n";
			break;
	}
	
	overflow_sign_tmp = (significant_result & 0xff800000) >> 23;
	dout_tmp = (significant_result << overflow_sign_tmp) & 0x007fffff;
	result_exp_tmp = fpub_exponent_tmp  + overflow_sign_tmp; 
	dout_tmp = dout_tmp | ((result_exp_tmp << 23) & 0x7f800000);
	result_sign_tmp = fpua_sign_tmp;
	dout_tmp = dout_tmp | ((result_sign_tmp << 31) & 0x80000000);
	dout = dout_tmp;
	out_valid = true;
	out_valid = false;
}

void floating_H::run()
{
	//---------------------------
	read_test_input();
	int c = 0;
	//----------------------------
	
	while(true) 
	{
		//------------------------
		//-----------------------
		run_floating();
		//------------------------
		c++;
		if(c >= 9)	break;
		//-----------------------
	}
}
