/*****************************************************************************

  The following code is derived, directly or indirectly, from the SystemC
  source code Copyright (c) 1996-2006 by all Contributors.
  All Rights reserved.

  The contents of this file are subject to the restrictions and limitations
  set forth in the SystemC Open Source License Version 2.4 (the "License");
  You may not use this file except in compliance with such restrictions and
  limitations. You may obtain instructions on how to receive a copy of the
  License at http://www.systemc.org/. Software distributed by Contributors
  under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
  ANY KIND, either express or implied. See the License for the specific
  language governing rights and limitations under the License.

 *****************************************************************************/

/*****************************************************************************
 
  registerfile_h.cpp -- Register File
 
  Original Author: Nicolas Blanc, ETHZ.
 
 *****************************************************************************/

/*****************************************************************************
 
  MODIFICATION LOG - modifiers, enter your name, affiliation, date and
  changes you are making here.
 
      Name, Affiliation, Date:
  Description of Modification:
 
 *****************************************************************************/

#include <systemc.h>
#include "registerfile_H.h"

//--------------------------------
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	8
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
	fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "i-" << test_data_in[c][i] << " ";
			}
			c++;
			// 			std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}
//--------------------------------

void registerfile_H::read1()
{
  if(rst==true)
  {
    data1 = 0;
  }
  else if(cs==true && re == true)
  {
    data1 = reg[addr1];
  }
// 	std::cout << "read1 " << addr1 << ":"
// 	<< reg[addr1] << ":"
// 	<< data1 << ":"
// 	<< std::endl;
}

void registerfile_H::read2()
{
  if(rst==true)
  {
    data2 = 0;
  }
  else if(cs==true && re == true)
  {
    data2 = reg[addr2];
  }
// 	std::cout << "read2 " << addr2 << ":"
// 	<< reg[addr2] << ":"
// 	<< data2 << ":"
// 	<< std::endl;
}

void registerfile_H::write3()
{
  if(rst==true)
  {}
  else if(cs==true && we == true)
  {
    reg[addr3] = data3;
  }
// 	std::cout << "write3 " << addr3 << ":" << data3 << ":" << reg[addr3] << std::endl;
}

void registerfile_H::flush()
{
  if(rst == true)
  {
    for(unsigned i = 0; i < 16; i++)
    {
      reg[i] = 0;
    }
  }
}

void registerfile_H::run()
{
	//---------------------------
	read_test_input();
	int c = 0;
	//----------------------------
	
	while(true)
	{
		//-------------------------
		rst = test_data_in[c][0];
		cs = test_data_in[c][1];
		re = test_data_in[c][2];
		we = test_data_in[c][3];
		addr1 = test_data_in[c][4];
		addr2 = test_data_in[c][5];
		addr3 = test_data_in[c][6];
		data3 = test_data_in[c][7];
		std::cout << sc_time_stamp() << "i-"
		<< rst << " "
		<< cs << " "
		<< re << " "
		<< we << " "
		<< addr1 << " "
		<< addr2 << " "
		<< addr3 << " "
		<< (unsigned)data3 << " "
		<< std::endl;
		//-------------------------
		
		read1();
		read2();
		write3();
		flush();
		
		//-------------------------
		std::cout << sc_time_stamp() << "o-"
		<< (unsigned)data1 << " "
		<< (unsigned)data2 << " "
		<< std::endl;
		if(test_data_out_x[c][0] == 0)
			assert((unsigned)(data1) == test_data_out[c][0]);
		if(test_data_out_x[c][1] == 0)
			assert((unsigned)(data2) == test_data_out[c][1]);
		c++;
		if(c >= 9)	break;
		//-------------------------
	}
}


