#include "systemc.h"

// BINARY ENCODED state machine: txWireArb
// State codes definitions:
#define START_TARB 0
#define TARB_WAIT_REQ 1
#define PTXB_ACT 2
#define SIE_TX_ACT 3

SC_MODULE(USBTxWireArbiter_H)
{
  bool		SIETxCtrl;
  unsigned	SIETxData;
  bool		SIETxFSRate;
  bool		SIETxReq;
  bool		SIETxWEn;
  bool		USBWireRdyIn;
  bool		prcTxByteCtrl;
  unsigned	prcTxByteData;
  bool		prcTxByteFSRate;
  bool		prcTxByteReq;
  bool		prcTxByteWEn;
  bool		rst;

  bool		SIETxGnt;
  unsigned	TxBits;
  bool		TxCtl;
  bool		TxFSRate;
  bool		USBWireRdyOut;
  bool		USBWireWEn;
  bool		prcTxByteGnt;
	
  void show();
  void txWireArb_mux();
  void txWireArb_arb(unsigned);
  void txWireArb(unsigned);
  void run();

  bool		muxSIENotPTXB;  
  unsigned	state_txWireArb;

  SC_CTOR(USBTxWireArbiter_H)
  {
    SC_THREAD(run);
  }
};


void USBTxWireArbiter_H::show()
{
  std::cout << SIETxCtrl << " " << SIETxData << " " << SIETxFSRate << " " << SIETxReq << " " 
	<< SIETxWEn << " " << USBWireRdyIn << " " << prcTxByteCtrl << " " << prcTxByteData << " " 
	<< prcTxByteFSRate << " " << prcTxByteReq << " " << prcTxByteWEn << " " << rst << " \t"
	<< SIETxGnt << " " << TxBits << " " << TxCtl << " " << TxFSRate << " " 
	<< USBWireRdyOut << " " << USBWireWEn << " " << prcTxByteGnt << " " 
	<< std::endl;
}

// processTxByte/SIETransmitter mux
void USBTxWireArbiter_H::txWireArb_mux()
{
  USBWireRdyOut = USBWireRdyIn;
}

void USBTxWireArbiter_H::txWireArb_arb(unsigned mux)
{
  if (mux == true)
  {
    USBWireWEn = SIETxWEn;
    TxBits = SIETxData;
    TxCtl = SIETxCtrl;
    TxFSRate = SIETxFSRate;
  }
  else
  {
    USBWireWEn = prcTxByteWEn;
    TxBits = prcTxByteData;
    TxCtl = prcTxByteCtrl;
    TxFSRate = prcTxByteFSRate;
  }
}

void USBTxWireArbiter_H::txWireArb(unsigned state)
{
  switch(state)
  {
    case START_TARB:
      state_txWireArb = TARB_WAIT_REQ;
      break;
    case TARB_WAIT_REQ:
      if (prcTxByteReq == true)	
      {
        state_txWireArb = PTXB_ACT;
        prcTxByteGnt = true;
        muxSIENotPTXB = false;
      }
      else if (SIETxReq == true)	
      {
        state_txWireArb = SIE_TX_ACT;
        SIETxGnt = true;
        muxSIENotPTXB = true;
      }
      break;
    case PTXB_ACT:
      if (prcTxByteReq == false)	
      {
        state_txWireArb = TARB_WAIT_REQ;
        prcTxByteGnt = false;
      }
      break;
    case SIE_TX_ACT:
      if (SIETxReq == false)	
      {
        state_txWireArb = TARB_WAIT_REQ;
        SIETxGnt = false;
      }
      break;
  }
}

//--------------------------------------------------------------------
// Machine: txWireArb
//--------------------------------------------------------------------
void USBTxWireArbiter_H::run()
{
  SIETxCtrl = false;
  SIETxData = 10;
  SIETxFSRate = false;
  SIETxReq = false;
  SIETxWEn = true;
  USBWireRdyIn = true;
  prcTxByteCtrl = true;
  prcTxByteData = 9;
  prcTxByteFSRate = false;
  prcTxByteReq = true;
  prcTxByteWEn = false;
  rst = false;

  while(true)
  {
    show();

    if (rst)
    {
      state_txWireArb = START_TARB;
      muxSIENotPTXB = 0;
      prcTxByteGnt = 0;
      SIETxGnt = 0;
    }
    else
    {
      txWireArb_mux();
      txWireArb_arb(muxSIENotPTXB);
      txWireArb(state_txWireArb);
    }
  }
}



