#include <iostream>
#include "wb_master_model_H.h"

//--------------------------------
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	8
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
	fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "i-" << test_data_in[c][i] << " ";
			}
			c++;
			// 			std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}
//--------------------------------

void wb_master_model_H::initializeMem()
{
  for(unsigned i=0; i < awidth; i++)    lv_a[i]="X";
  for(unsigned i=0; i < dwidth; i++)    lv_d[i]="X";

  adr=lv_a;
  dout=lv_d;
  cyc=false;
  stb="X";
  we="X";
  sel=lv_d.range(dwidth/8 - 1, 0);
  // nannan
  delay_nan=7;
}


void wb_master_model_H::wb_write_task(unsigned delay, sc_lv<awidth -1> a, sc_lv<dwidth -1> d)
{
  adr=a;
  dout=d;
  cyc=true;
  stb=true;
  we=true;

  while(true)
  {
    if(ack == true)
      break;
  }

  for(unsigned i=0; i < dwidth; i++)    lv_d[i]="1";
  sel=lv_d.range(dwidth/8 - 1, 0);

  cyc=false;
  stb="X";
  we="X";

  for(unsigned i=0; i < awidth; i++)    lv_a[i]="X";
  for(unsigned i=0; i < dwidth; i++)    lv_d[i]="X";

  adr=lv_a;
  dout=lv_d;
  sel=lv_d.range(dwidth/8 - 1, 0);
}

sc_lv<dwidth -1>  wb_master_model_H::wb_read_task(unsigned delay, sc_lv<awidth -1> a)
{
  unsigned i=0;
  while(i < delay)
  {
    i++;
  }

  { 
    adr=a;

    for(unsigned i=0; i < dwidth; i++)    lv_d[i]="X";
    dout=lv_d;

    cyc=true;
    stb=true;
    we=true;
  }

  while(true)
  {
    if(ack == true)
      break;
  }

  for(unsigned i=0; i<dwidth; i++)    lv_d[i]="1";
  sel.range(dwidth/8-1, 0);

  cyc=false;
  stb="X";
  we="X";

  for(unsigned i=0; i < awidth; i++)    lv_a[i]="X";
  for(unsigned i=0; i < dwidth; i++)    lv_d[i]="X";

  adr=lv_a;
  dout=lv_d;
  sel=lv_d.range(dwidth/8-1, 0);
  return din;
}

void wb_master_model_H::wb_cmp_task(unsigned delay, sc_lv<awidth-1> a, sc_lv<dwidth-1> d_exp)
{
  q=wb_read_task(delay, a);

  if(d_exp!=q)
  {
    cout << "\n--- ERROR: At address 0x" << a << ",got 0x" << 
      q << ", expected 0x" << d_exp << "at time " << sc_time_stamp() << endl;
    sc_stop();
  }
}

void wb_master_model_H::call_task()
{
  wb_cmp_task(delay_nan, a_nan, d_exp_nan);
}

