// v2sc benchmarks
// Nannan He
// X/XX/9

#ifndef USB_SERIAL_INTERFACE_ENGINE_H
#define USB_SERIAL_INTERFACE_ENGINE_H

 // Sampling frequency = 'FS_OVER_SAMPLE_RATE' * full speed bit rate = 'LS_OVER_SAMPLE_RATE' * low speed bit rate
#define FS_OVER_SAMPLE_RATE 4
#define LS_OVER_SAMPLE_RATE 32

//timeOuts
#define RX_PACKET_TOUT 18
#define RX_EDGE_DET_TOUT 7

//TXStreamControlTypes
#define TX_DIRECT_CONTROL 0x00
#define TX_RESUME_START 0x01
#define TX_PACKET_START 0x02
#define TX_PACKET_STREAM 0x03
#define TX_PACKET_STOP 0x04
#define TX_IDLE 0x05
#define TX_LS_KEEP_ALIVE 0x06

//RXStreamControlTypes
#define RX_PACKET_START 0
#define RX_PACKET_STREAM 1
#define RX_PACKET_STOP 2

//USBLineStates
// ONE_ZERO corresponds to differential 1. ie D+ = Hi, D- = Lo
#define ONE_ZERO 0b10
#define ZERO_ONE 0b01
#define SE0 0b00
#define SE1 0b11

//RXStatusIndices
#define CRC_ERROR_BIT 0
#define BIT_STUFF_ERROR_BIT 1
#define RX_OVERFLOW_BIT 2
#define NAK_RXED_BIT 3
#define STALL_RXED_BIT 4
#define ACK_RXED_BIT 5
#define DATA_SEQUENCE_BIT 6

//usbWireControlStates
#define TRI_STATE 0b0
#define DRIVE 0b1

//limits
#define MAX_CONSEC_SAME_BITS 0x6
#define MAX_CONSEC_SAME_BITS_PLUS1 0x7
// RESUME_RX_WAIT_TIME defines the time period for resume detection
// The resume counter is incremented at the bit rate, so
// RESUME_RX_WAIT_TIME = 29 corresponds to 30 * 1/12MHz = 2.5uS at full speed
// and 30 * 1/1.5MHz =  20uS at low speed, both of which are within the USB spec of 
// 2.5uS <= resumeDetectTime <= 100uS
#define RESUME_RX_WAIT_TIME 0d29
//#define RESUME_WAIT_TIME_MINUS1 9
// 'HOST_TX_RESUME_TIME' assumes counter is incremented at low speed bit rate 
#ifdef SIM_COMPILE 
#define HOST_TX_RESUME_TIME 0d10
#else
#define HOST_TX_RESUME_TIME 0d30000  //Host sends resume for 30000 * 1/1.5MHz = 20mS
#endif
//#define CONNECT_WAIT_TIME 0d20
#define CONNECT_WAIT_TIME 0d120      //Device connect detected after 120 * 1/48MHz = 2.5uS
//#define DISCONNECT_WAIT_TIME 0d20   
#define DISCONNECT_WAIT_TIME 0d120   //Device disconnect detected after 120 * 1/48MHz = 2.5uS

//RXConnectStates
#define DISCONNECT	0
#define LOW_SPEED_CONNECT	1
#define FULL_SPEED_CONNECT	2

//TX_RX_InternalStreamTypes
#define DATA_START 0x00
#define DATA_STOP 0x01
#define DATA_STREAM 0x02
#define DATA_BIT_STUFF_ERROR 0x03

//RXStMach states
#define DISCONNECT_ST 0x0
#define WAIT_FULL_SPEED_CONN_ST 0x1
#define WAIT_LOW_SPEED_CONN_ST 0x2
#define CONNECT_LOW_SPEED_ST 0x3
#define CONNECT_FULL_SPEED_ST 0x4
#define WAIT_LOW_SP_DISCONNECT_ST 0x5
#define WAIT_FULL_SP_DISCONNECT_ST 0x6

//RXBitStateMachStates
#define IDLE_BIT_ST	0
#define DATA_RECEIVE_BIT_ST	1
#define WAIT_RESUME_ST	2
#define RESUME_END_WAIT_ST	3

//RXByteStateMachStates 
#define IDLE_BYTE_ST	0
#define CHECK_SYNC_ST	1
#define CHECK_PID_ST	2
#define HS_BYTE_ST	3
#define TOKEN_BYTE_ST	4
#define DATA_BYTE_ST	5

#endif
