//////////////////////////////////////////////////////////////////////
////                                                              ////
////  Mixcolumns for a 16 bit word module implementation          ////
////                                                              ////
////  This file is part of the SystemC AES                        ////
////                                                              ////
////  Description:                                                ////
////  Mixcolum for a 16 bit word                                  ////
////                                                              ////
////  To Do:                                                      ////
////   - done                                                     ////
////                                                              ////
////  Author(s):                                                  ////
////      - Javier Castillo, jcastilo@opencores.org               ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2000 Authors and OPENCORES.ORG                 ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE.  See the GNU Lesser General Public License for more ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from http://www.opencores.org/lgpl.shtml                     ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
// CVS Revision History
//
// Format revision  2009-9-3	nannan
//
// $Log: word_mixcolum_H.cpp,v $
// Revision 1.1  2007-10-11 17:23:52  blancn
// *** empty log message ***
//
// Revision 1.1  2005/02/14 11:18:31  jcastillo
// Moved
//
// Revision 1.3  2005/01/20 18:14:06  jcastillo
// Style changes to fit sc2v
//
// Revision 1.2  2004/08/30 14:44:44  jcastillo
// Code Formater used to give better appearance to SystemC code
//
// Revision 1.1.1.1  2004/07/05 09:46:22  jcastillo
// First import
//

#include "word_mixcolum_H.h"

//--------------------------------
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	8
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
	fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "i-" << test_data_in[c][i] << " ";
			}
			c++;
			// 			std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}
//--------------------------------

void word_mixcolum_H::init()
{
  bm1 = new byte_mixcolum_H("bm1");
  bm2 = new byte_mixcolum_H("bm2");
  bm3 = new byte_mixcolum_H("bm3");
  bm4 = new byte_mixcolum_H("bm4");
}

void word_mixcolum_H::update()
{
  bm1->a = a;
  bm1->b = b;
  bm1->c = c;
  bm1->d = d;
  x1 = bm1->outx;
  y1 = bm1->outy;

  bm2->a = a;
  bm2->b = b;
  bm2->c = c;
  bm2->d = d;
  x2 = bm2->outx;
  y2 = bm2->outy;

  bm3->a = a;
  bm3->b = b;
  bm3->c = c;
  bm3->d = d;
  x3 = bm3->outx;
  y3 = bm3->outy;

  bm4->a = a;
  bm4->b = b;
  bm4->c = c;
  bm4->d = d;
  x4 = bm4->outx;
  y4 = bm4->outy;
}

void word_mixcolum_H::run()
{
  init();
  while(true) 
  {
    update();
    split();
    mix();
  }
}

void word_mixcolum_H::mix()
{
  sc_uint<32> outx_var, outy_var;

  outx_var.range(31, 24) = x1;
  outx_var.range(23, 16) = x2;
  outx_var.range(15, 8) = x3;
  outx_var.range(7, 0) = x4;
  outy_var.range(31, 24) = y1;
  outy_var.range(23, 16) = y2;
  outy_var.range(15, 8) = y3;
  outy_var.range(7, 0) = y4;

  outx = outx_var;
  outy = outy_var;
}

void word_mixcolum_H::split()
{
  sc_uint<32> in_var;

  in_var = in;
  a = in_var.range(31, 24);
  b = in_var.range(23, 16);
  c = in_var.range(15, 8);
  d = in_var.range(7, 0);
}

