//////////////////////////////////////////////////////////////////////
////                                                              ////
////  Word mixcolum header                                        ////
////                                                              ////
////  This file is part of the SystemC AES                        ////
////                                                              ////
////  Description:                                                ////
////  Header file for 16-bit mixcolum submodule                   ////
////                                                              ////
////  To Do:                                                      ////
////   - done                                                     ////
////                                                              ////
////  Author(s):                                                  ////
////      - Javier Castillo, jcastilo@opencores.org               ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2000 Authors and OPENCORES.ORG                 ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE.  See the GNU Lesser General Public License for more ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from http://www.opencores.org/lgpl.shtml                     ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
// CVS Revision History
//
// Format revision  2009-9-3	nannan
//
// $Log: word_mixcolum_H.h,v $
// Revision 1.1  2007-10-11 17:23:52  blancn
// *** empty log message ***
//
// Revision 1.1  2005/02/14 11:18:31  jcastillo
// Moved
//
// Revision 1.3  2005/01/20 18:14:06  jcastillo
// Style changes to fit sc2v
//
// Revision 1.2  2004/08/30 14:44:44  jcastillo
// Code Formater used to give better appearance to SystemC code
//
// Revision 1.1.1.1  2004/07/05 09:46:22  jcastillo
// First import
//

#include "systemc.h"
#include "byte_mixcolum_H.h"

SC_MODULE(word_mixcolum_H)
{
  sc_uint<32> in;
  sc_uint<32> outx, outy;

  void run();
  void init();

  void update();
  void split();
  void mix();

  sc_uint<8> a, b, c, d;
  sc_uint<8> x1, x2, x3, x4, y1, y2, y3, y4;
	
  byte_mixcolum_H *bm1;
  byte_mixcolum_H *bm2;
  byte_mixcolum_H *bm3;
  byte_mixcolum_H *bm4;

  SC_CTOR(word_mixcolum_H)
  {
    SC_THREAD(run);
  }
};

void word_mixcolum_H::init()
{
  bm1 = new byte_mixcolum_H("bm1");
  bm2 = new byte_mixcolum_H("bm2");
  bm3 = new byte_mixcolum_H("bm3");
  bm4 = new byte_mixcolum_H("bm4");
}

void word_mixcolum_H::update()
{
  bm1->a = a;
  bm1->b = b;
  bm1->c = c;
  bm1->d = d;
  x1 = bm1->outx;
  y1 = bm1->outy;

  bm2->a = a;
  bm2->b = b;
  bm2->c = c;
  bm2->d = d;
  x2 = bm2->outx;
  y2 = bm2->outy;

  bm3->a = a;
  bm3->b = b;
  bm3->c = c;
  bm3->d = d;
  x3 = bm3->outx;
  y3 = bm3->outy;

  bm4->a = a;
  bm4->b = b;
  bm4->c = c;
  bm4->d = d;
  x4 = bm4->outx;
  y4 = bm4->outy;
}

void word_mixcolum_H::run()
{
  init();
  while(true) 
  {
    update();
    split();
    mix();
  }
}

void word_mixcolum_H::mix()
{
  sc_uint<32> outx_var, outy_var;

  outx_var.range(31, 24) = x1;
  outx_var.range(23, 16) = x2;
  outx_var.range(15, 8) = x3;
  outx_var.range(7, 0) = x4;
  outy_var.range(31, 24) = y1;
  outy_var.range(23, 16) = y2;
  outy_var.range(15, 8) = y3;
  outy_var.range(7, 0) = y4;

  outx = outx_var;
  outy = outy_var;
}

void word_mixcolum_H::split()
{
  sc_uint<32> in_var;

  in_var = in;
  a = in_var.range(31, 24);
  b = in_var.range(23, 16);
  c = in_var.range(15, 8);
  d = in_var.range(7, 0);
}


